%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "taula.h"
//#include "global.h"
#include <stdarg.h>

FILE *logsSintactic;
extern int lineNumber;
extern int colNumber;
void sinLog(const char *format, ...);
void sinError(const char *format, ...);
extern Taula_Simbols *taulaSimbolActual, *taulaSimbolPrincipal;

Cua *cua_tmp, *cua_strings_tmp, *cua_defining_identifier_tmp;
int num_elements_cua_strings_tmp;
Taula_Simbols *taula_tmp;
int contador;  // Utilitzat per contar enumerats a la definicio, parametres a la crida de funcions/procediments
Info_Function *infoFunctionActual;  // Apunta al info_function de la funció que esta sent tratada. Només s'accedeix quan es troba un RETURN. Util per quan es troba un RETURN al cos, es pugui marcar el simbol de la funció a TRUE

Simbol *cercar_simbol_a_cua(Cua *cua, char *nom);
Taula_Simbols *intentar_duplicar_taula_simbols (Taula_Simbols *ts);
Boolean parametres_diferents(Cua *strings_parametres1, Cua *strings_parametres2, Taula_Simbols *parametres1, Taula_Simbols *parametres2);
Boolean crida_parametres_diferents(Cua *valors, Cua *strings_parametres, Taula_Simbols *parametres);
void comprovacio_complets(Taula_Simbols *ts);
void sinLog(const char *format, ...);
void sinError(const char *format, ...);

%}

%union {
	// Inicialitzat des de flex
    Simbol *simbol_ptr; 		// Si el simbol de l'identificador ja existeix, el referenciem
	Simbol simbol; 				// Valors que pasem cap adalt en l'arbre sintactic (bison).
    double  vfloat; 	// Valor numeric decimal
    int     vinteger;		// Valor numeric senser
    char    string[CONTENIDOR_STRING_LENGTH];	// Valor caracters, strings o nom de nou identificador
	char    character;	// Valor caracters, strings o nom de nou identificador
}

%token <string>	PR_ID
%token <simbol_ptr>		PR_ID_VARIABLE
%token <simbol_ptr>		PR_ID_FUNCTION
%token <simbol_ptr>		PR_ID_PROCEDURE
%token <simbol_ptr>		PR_ID_TYPE

%token <vinteger>  	PR_NUMERIC_LITERAL
%token <vfloat>		PR_DECIMAL_LITERAL
%token <character>    	PR_CHARACTER_LITERAL
%token <string>       PR_STRING_LITERAL

// El lexic no retornara cap valor als seguents tokens
%token <simbol_ptr>        PR_ABS
%token <simbol_ptr>        PR_ACCESS
%token <simbol_ptr>        PR_ALL
%token <simbol_ptr>        PR_AND
%token <simbol_ptr>        PR_ARRAY
%token <simbol_ptr>        PR_BEGIN
%token <simbol_ptr>        PR_CASE
%token <simbol_ptr>        PR_CONSTANT
%token <simbol_ptr>        PR_DECLARE
%token <simbol_ptr>        PR_IS
%token <simbol_ptr>        PR_ELSE
%token <simbol_ptr>        PR_ELSIF
%token <simbol_ptr>        PR_END
%token <simbol_ptr>        PR_EXIT
%token <simbol_ptr>        PR_FOR
%token <simbol_ptr>        PR_FUNCTION
%token <simbol_ptr>        PR_IF
%token <simbol_ptr>        PR_IN
%token <simbol_ptr>        PR_LOOP
%token <simbol_ptr>        PR_MOD
%token <simbol_ptr>        PR_NEW
%token <simbol_ptr>        PR_NOT
%token <simbol_ptr>        PR_NULL
%token <simbol_ptr>        PR_OF
%token <simbol_ptr>        PR_OR
%token <simbol_ptr>        PR_OTHERS
%token <simbol_ptr>        PR_PROCEDURE
%token <simbol_ptr>        PR_RANGE
%token <simbol_ptr>        PR_RECORD
%token <simbol_ptr>        PR_RETURN
%token <simbol_ptr>        PR_REVERSE
%token <simbol_ptr>        PR_SUBTYPE
%token <simbol_ptr>        PR_THEN
%token <simbol_ptr>        PR_TYPE
%token <simbol_ptr>        PR_WHEN
%token <simbol_ptr>        PR_WHILE

%token <simbol_ptr>	    PR_EQU
%token <simbol_ptr>    	PR_ARROW
%token <simbol_ptr>    	PR_MAJEQU
%token <simbol_ptr>    	PR_MINEQU
%token <simbol_ptr>		PR_DIFFERENT
%token <simbol_ptr>    	PR_EXP
%token <simbol_ptr>		PR_2_POINTS

// Reduce/Shift expected conflicts
%expect 2

%left '-' '+'
%left '*' '/'
%right PR_EXP

%left PR_IF
%left PR_THEN
%left PR_ELSE
%left PR_ELSIF


%type <simbol> type_definition enumeration_type_definition enumeration_literal_specification record_definition record_type_definition range direct_name name primary factor term simple_expression component_definition range_constraint scalar_constraint subtype_mark subtype_indication defining_identifier_element constraint indexed_component prefix relation expression selected_component type_conversion function_call array_type_definition integer_type_definition signed_integer_type_definition constrained_array_definition discrete_subtype_definition parameter_and_result_profile subprogram_specification procedure_call_statement designator subprogram_body_element program_element condition
%type <string> defining_identifier defining_character_literal selector_name defining_program_unit_name defining_designator
%type <vinteger> actual_parameter_part_element function_call_element

%start program

%{
    //#define YYERROR_VERBOSE 1
%}

%%

basic_declaration:
   type_declaration { sinLog(" -> basic_declaration -> type_declaration"); } 
   | object_declaration {sinLog(" -> basic_declaration -> object_declaration");} 
   | subprogram_declaration {sinLog(" -> basic_declaration -> subprogram_declaration");}
;

subprogram_declaration:
   subprogram_specification ';' {sinLog(" -> subprogram_declaration -> subprogram_specification");}
;

subprogram_specification:
   // Inicialitzar taula simbols entre PR_PROCEDURE i defining_program_unit_name (nou ambit)
   // Controlar errors de procedures/functions ja declarats o incomplets (són valids)
   // El final de la branca de 'defining_program_unit_name' és un PR_ID
   PR_PROCEDURE defining_program_unit_name {
	   // Nova definicio
	   taula_tmp = (Taula_Simbols *)crear_taula_simbols($2, taulaSimbolActual);
	   cua_strings_tmp = (Cua *)crear_cua();
	   num_elements_cua_strings_tmp = 0;
   }
   parameter_profile {
	   sinLog("subprogram_specification -> PR_PROCEDURE defining_program_unit_name parameter_profile");
	   Simbol *sym = crear_i_inserir_en_taula(taulaSimbolActual, taula_tmp->nom);
	   sym->atributs.tipus = PROCEDURE;
	   sym->atributs.continguts.info_procedure.id = 0;
	   sym->atributs.continguts.info_procedure.num_sobrecarregues = 0;
	   sym->atributs.continguts.info_procedure.complet = FALSE;
	   sym->atributs.continguts.info_procedure.num_parametres = num_elements_cua_strings_tmp;
	   num_elements_cua_strings_tmp = 0;
	   sym->atributs.continguts.info_procedure.parametres = cua_strings_tmp;
	   cua_strings_tmp = NULL;
	   sym->atributs.continguts.info_procedure.taula = taula_tmp;
	   taula_tmp = NULL;
	   sym->atributs.continguts.info_procedure.sobrecarregues = NULL;
	   
	   // Tornem el nou simbol
	   memcpy(&$$, sym, sizeof(struct Simbol));
   }
   | PR_PROCEDURE PR_ID_PROCEDURE {
	   // NOTE: Possible redefinicio o sobrecarrega
	   Info_Procedure *info_procedure = (Info_Procedure*)malloc(sizeof(struct Info_Procedure)); 
	   taula_tmp = (Taula_Simbols *)crear_taula_simbols($2, taulaSimbolActual);
	   cua_strings_tmp = (Cua *)crear_cua();
	   num_elements_cua_strings_tmp = 0;
   }
   parameter_profile {
	   sinLog("subprogram_specification -> PR_PROCEDURE PR_ID_PROCEDURE parameter_profile");
	   Boolean trobada_sobrecarrega_definida = FALSE;
	   Info_Procedure *info_procedure_sobrecarregat = &($2->atributs.continguts.info_procedure); // Apuntara al info de la sobrecarrega ja definida en cas d'existir
	   int total_sobrecarregues = info_procedure_sobrecarregat->num_sobrecarregues;
	   
	   // Busquem el Info_Function amb mateix parametres que el procedure trobat per la regla
	   // 1.- Comparem amb el Info_Function principal
	   if (num_elements_cua_strings_tmp == info_procedure_sobrecarregat->num_parametres) {
		   // Num. parametres iguals
		   if (parametres_diferents(info_procedure_sobrecarregat->parametres, cua_strings_tmp, info_procedure_sobrecarregat->taula, taula_tmp) == FALSE) {
			   // Tipus parametres iguals
			   trobada_sobrecarrega_definida = TRUE;
		   }
	   }

	   // 2.- Comparem amb els Info_Functions del llistat de sobrecarregues
	   if (trobada_sobrecarrega_definida == FALSE) {
		   if ($2->atributs.continguts.info_procedure.sobrecarregues != NULL) {
			   reiniciar_PDI($2->atributs.continguts.info_procedure.sobrecarregues);
			   info_procedure_sobrecarregat = (Info_Procedure *)obtenir_element_PDI($2->atributs.continguts.info_procedure.sobrecarregues);		   
			   while (info_procedure_sobrecarregat != NULL) {
				   if (num_elements_cua_strings_tmp == info_procedure_sobrecarregat->num_parametres) {
					   // Num. parametres iguals
					   if (parametres_diferents(info_procedure_sobrecarregat->parametres, cua_strings_tmp, info_procedure_sobrecarregat->taula, taula_tmp) == FALSE) {
						   // Tipus parametres iguals
						   trobada_sobrecarrega_definida = TRUE;
						   break; // No cal continuar buscant
					   }
				   }
				   seguent_PDI($2->atributs.continguts.info_procedure.sobrecarregues);
				   info_procedure_sobrecarregat = (Info_Procedure *)obtenir_element_PDI($2->atributs.continguts.info_procedure.sobrecarregues);
			   }
		   }
	   }
	   
	   // Si no hem trobat cap sobrecarrega igual
	   if (trobada_sobrecarrega_definida == FALSE) {
		   // Afegim nova sobrecàrrega
		   if ($2->atributs.continguts.info_procedure.sobrecarregues == NULL) {
			   $2->atributs.continguts.info_procedure.sobrecarregues = (Cua *)crear_cua();
		   }
		   $2->atributs.continguts.info_procedure.num_sobrecarregues++;
		   
		   Info_Procedure *info_procedure = (Info_Procedure*)malloc(sizeof(struct Info_Procedure));
		   info_procedure->id = total_sobrecarregues + 1;
		   info_procedure->complet = FALSE;
		   info_procedure->num_parametres = num_elements_cua_strings_tmp;
		   num_elements_cua_strings_tmp = 0;
		   info_procedure->parametres = cua_strings_tmp;
		   cua_strings_tmp = NULL;
		   info_procedure->taula = taula_tmp;
		   taula_tmp = NULL;
		   
		   encuar($2->atributs.continguts.info_procedure.sobrecarregues, info_procedure);
		   
		   // Retornem el simbol original amb els Info_Function de la sobrecarrega nova
		   memcpy(&$$, $2, sizeof(struct Simbol));
		   memcpy(&($$.atributs.continguts.info_procedure), info_procedure , sizeof(struct Info_Procedure));
	   } else {
		   // Hem trobat la sobrecarrega ya definida, no cal definir-la de nou
		   // TODO: Alliberar taula_tmp i cua_strings_tmp
		   // Retornem el simbol original amb els Info_Function de la sobrecarrega trobada
		   memcpy(&$$, $2, sizeof(struct Simbol));
		   memcpy(&($$.atributs.continguts.info_procedure), info_procedure_sobrecarregat , sizeof(struct Info_Procedure));
	   }
   }
   | PR_FUNCTION defining_designator {
	   taula_tmp = (Taula_Simbols *)crear_taula_simbols($2, taulaSimbolActual);
	   cua_strings_tmp = (Cua *)crear_cua();
	   num_elements_cua_strings_tmp = 0;
   }
   parameter_and_result_profile {
	   sinLog("subprogram_specification -> PR_PROCEDURE defining_program_unit_name parameter_and_result_profile");
	   Simbol *sym = crear_i_inserir_en_taula(taulaSimbolActual, taula_tmp->nom);
	   sym->atributs.tipus = FUNCTION;
	   sym->atributs.continguts.info_function.id = 0;
	   sym->atributs.continguts.info_function.num_sobrecarregues = 0;
	   sym->atributs.continguts.info_function.complet = FALSE;
	   sym->atributs.continguts.info_function.return_indicat = FALSE;
	   sym->atributs.continguts.info_function.num_parametres = num_elements_cua_strings_tmp;
	   num_elements_cua_strings_tmp = 0;
	   sym->atributs.continguts.info_function.parametres = cua_strings_tmp;
	   cua_strings_tmp = NULL;
	   sym->atributs.continguts.info_function.taula = taula_tmp;
	   taula_tmp = NULL;
	   sym->atributs.continguts.info_function.sobrecarregues = NULL;
	   
	   // $4 és 'parameter_and_result_profile'
	   if ($4.atributs.tipus == TYPE) {
			// Tipus bàsic
			sym->atributs.continguts.info_function.tipus_retorn = $4.atributs.continguts.info_type.tipus;
			if ($4.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT) {
				memcpy(sym->atributs.continguts.info_function.nom_tipus_retorn, $4.nom, STRING_LENGTH);
			}
	   } else if ($4.atributs.tipus == ENUMERATED_TYPE) {
			// A partir del nom podem extreure el simbol del tipus
			sym->atributs.continguts.info_function.tipus_retorn = TIPUS_DERIVAT;
			memcpy(sym->atributs.continguts.info_function.nom_tipus_retorn, $4.nom, STRING_LENGTH);
	   } else if ($4.atributs.tipus == RECORD_TYPE) {
			sym->atributs.continguts.info_function.tipus_retorn = TIPUS_DERIVAT;
			memcpy(sym->atributs.continguts.info_function.nom_tipus_retorn, $4.nom, STRING_LENGTH);
	   } else if ($4.atributs.tipus == ARRAY_TYPE) {
			sym->atributs.continguts.info_function.tipus_retorn = TIPUS_DERIVAT;
			if (strcmp($4.nom, "") == 0) {
				// NOTE: Considerem que no poden haver-hi tipus anonims a la signatura.
				sinError("No poden haver-hi tipus anonims a la signatura.");
			} else {
				// Array ya definit
				memcpy(sym->atributs.continguts.info_function.nom_tipus_retorn, $4.nom, STRING_LENGTH);
			}
	   } else {
			sinError("Sense tipus vàlid de retorn! %i", $4.atributs.tipus);
	   }
	   
	   // Util per quan es troba un RETURN al cos, es pugui marcar el simbol de la funció a TRUE
	   infoFunctionActual = &(sym->atributs.continguts.info_function);
	   
	   // Tornem el nou simbol
	   memcpy(&$$, sym, sizeof(struct Simbol));
   }
   | PR_FUNCTION PR_ID_FUNCTION {
	   // NOTE: Posible redefinicio o sobrecarrega
	   taula_tmp = (Taula_Simbols *)crear_taula_simbols($2, taulaSimbolActual);
	   cua_strings_tmp = (Cua *)crear_cua();
	   num_elements_cua_strings_tmp = 0;
   }
   parameter_and_result_profile {
	   sinLog("subprogram_specification -> PR_FUNCTION PR_ID_FUNCTION parameter_and_result_profile");
	   Boolean trobada_sobrecarrega_definida = FALSE;
	   Info_Function *info_function_sobrecarregat = &($2->atributs.continguts.info_function); // Apuntara al info de la sobrecarrega ja definida en cas d'existir
	   int total_sobrecarregues = info_function_sobrecarregat->num_sobrecarregues;
	   
	   // Busquem el info_function amb mateix parametres que la function trobada per la regla
	   // 1.- Comparem amb el info_function principal
	   if (num_elements_cua_strings_tmp == info_function_sobrecarregat->num_parametres) {
		   // Num. parametres iguals
		   if (parametres_diferents(info_function_sobrecarregat->parametres, cua_strings_tmp, info_function_sobrecarregat->taula, taula_tmp) == FALSE) {
			   // Tipus parametres iguals
			   trobada_sobrecarrega_definida = TRUE;
			   
			   // NOTE: No suportem sobrecarrega de tipus de retorn
			   // El tipus retornat ha de ser igual, sino generem error
			   if ($4.atributs.tipus == TYPE) {
				   if ($4.atributs.continguts.info_type.tipus == info_function_sobrecarregat->tipus_retorn) {
					   if ($4.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT) {
						   if (strcmp($4.nom, info_function_sobrecarregat->nom_tipus_retorn) != 0) {
							   // Tipus retorn no iguals
							   sinError("El tipus de retorn de la funcio no coincideix amb funcio '%s' anteriorment definida.", $2->nom);
						   }
					   }
				   } else {
					   sinError("El tipus de retorn de la funcio no coincideix amb funcio '%s' anteriorment definida.", $2->nom);
				   }
			   } else if (($4.atributs.tipus == ENUMERATED_TYPE) || ($4.atributs.tipus == RECORD_TYPE)
			   				|| ($4.atributs.tipus == ARRAY_TYPE)) {
				   		if (strcmp($4.nom, info_function_sobrecarregat->nom_tipus_retorn) != 0) {
							// Tipus retorn no iguals
							   sinError("El tipus de retorn de la funcio no coincideix amb funcio '%s' anteriorment definida.", $2->nom);
						}
			   } else {
					   sinError("Sense tipus vàlid de retorn! %i", $4.atributs.tipus);
			   }
			   
			   
			   // Comprovar tipus retorn
			   /*if ($4.atributs.tipus == TYPE) {
				   if ($4.atributs.continguts.info_type.tipus == info_function_sobrecarregat->tipus_retorn) {
					   if ($4.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT) {
						   if (strcmp($4.nom, info_function_sobrecarregat->nom_tipus_retorn) == 0) {
							   // Tipus retorn iguals
							   trobada_sobrecarrega_definida = TRUE;
						   }
					   } else {
						   // Tipus retorn iguals
						   trobada_sobrecarrega_definida = TRUE;
					   }
				   }
			   } else if (($4.atributs.tipus == ENUMERATED_TYPE) || ($4.atributs.tipus == RECORD_TYPE)
			   				|| ($4.atributs.tipus == ARRAY_TYPE)) {
				   		if (strcmp($4.nom, info_function_sobrecarregat->nom_tipus_retorn) == 0) {
							// Tipus retorn iguals
							trobada_sobrecarrega_definida = TRUE;
						}
			   } else {
					   sinError("Sense tipus vàlid de retorn! %i", $4.atributs.tipus);
			   }*/
		   }
	   }

	   // 2.- Comparem amb els info_function del llistat de sobrecarregues
	   if (trobada_sobrecarrega_definida == FALSE) {
		   if ($2->atributs.continguts.info_function.sobrecarregues != NULL) {
			   reiniciar_PDI($2->atributs.continguts.info_function.sobrecarregues);
			   info_function_sobrecarregat = (Info_Function *)obtenir_element_PDI($2->atributs.continguts.info_function.sobrecarregues);		   
			   while (info_function_sobrecarregat != NULL) {
				   if (num_elements_cua_strings_tmp == info_function_sobrecarregat->num_parametres) {
					   // Num. parametres iguals
					   
					   if (parametres_diferents(info_function_sobrecarregat->parametres, cua_strings_tmp, info_function_sobrecarregat->taula, taula_tmp) == FALSE) {
						   // Tipus parametres iguals
						   trobada_sobrecarrega_definida = TRUE;
			   
						   // NOTE: No suportem sobrecarrega de tipus de retorn
						   // El tipus retornat ha de ser igual, sino generem error
						   if ($4.atributs.tipus == TYPE) {
							   if ($4.atributs.continguts.info_type.tipus == info_function_sobrecarregat->tipus_retorn) {
								   if ($4.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT) {
									   if (strcmp($4.nom, info_function_sobrecarregat->nom_tipus_retorn) != 0) {
										   // Tipus retorn no iguals
										   sinError("El tipus de retorn de la funcio no coincideix amb funcio '%s' anteriorment definida.", $2->nom);
									   }
								   }
							   } else {
								   sinError("El tipus de retorn de la funcio no coincideix amb funcio '%s' anteriorment definida.", $2->nom);
							   }
						   } else if (($4.atributs.tipus == ENUMERATED_TYPE) || ($4.atributs.tipus == RECORD_TYPE)
										|| ($4.atributs.tipus == ARRAY_TYPE)) {
									if (strcmp($4.nom, info_function_sobrecarregat->nom_tipus_retorn) != 0) {
										// Tipus retorn no iguals
										   sinError("El tipus de retorn de la funcio no coincideix amb funcio '%s' anteriorment definida.", $2->nom);
									}
						   } else {
								   sinError("El tipus de retorn de la funcio no coincideix amb funcio '%s' anteriorment definida.", $2->nom);
						   }
						   
						   // NOTE: No suportem sobrecarrega de tipus de retorn
						   /*// Comprovar tipus retorn
						   if ($4.atributs.tipus == TYPE) {
							   if ($4.atributs.continguts.info_type.tipus == info_function_sobrecarregat->tipus_retorn) {
								   if ($4.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT) {
									   if (strcmp($4.nom, info_function_sobrecarregat->nom_tipus_retorn) == 0) {
										   // Tipus retorn iguals
										   trobada_sobrecarrega_definida = TRUE;
										   break; // No cal continuar buscant
									   }
								   } else {
									   // Tipus retorn iguals
									   trobada_sobrecarrega_definida = TRUE;
									   break; // No cal continuar buscant
								   }
							   }
						   } else if (($4.atributs.tipus == ENUMERATED_TYPE) || ($4.atributs.tipus == RECORD_TYPE)
										|| ($4.atributs.tipus == ARRAY_TYPE)) {
									if (strcmp($4.nom, info_function_sobrecarregat->nom_tipus_retorn) == 0) {
										// Tipus retorn iguals
										trobada_sobrecarrega_definida = TRUE;
										break; // No cal continuar buscant
									}
						   } else {
								   sinError("Sense tipus vàlid de retorn! %i", $4.atributs.tipus);
						   }*/
						   
					   }
				   }
				   seguent_PDI($2->atributs.continguts.info_function.sobrecarregues);
				   info_function_sobrecarregat = (Info_Function *)obtenir_element_PDI($2->atributs.continguts.info_function.sobrecarregues);
			   }
		   }
	   }
	   
	   // Si no hem trobat cap sobrecarrega igual
	   if (trobada_sobrecarrega_definida == FALSE) {
		   // Afegim nova sobrecàrrega
		   if ($2->atributs.continguts.info_function.sobrecarregues == NULL) {
			   $2->atributs.continguts.info_function.sobrecarregues = (Cua *)crear_cua();
		   }
		   
		   $2->atributs.continguts.info_function.num_sobrecarregues++;
		   
		   Info_Function *info_function = (Info_Function*)malloc(sizeof(struct Info_Function));
		   info_function->id = total_sobrecarregues + 1;
		   info_function->complet = FALSE;
		   info_function->return_indicat = FALSE;
		   info_function->num_parametres = num_elements_cua_strings_tmp;
		   num_elements_cua_strings_tmp = 0;
		   info_function->parametres = cua_strings_tmp;
		   cua_strings_tmp = NULL;
		   info_function->taula = taula_tmp;
		   taula_tmp = NULL;
		   
		   // $4 és 'parameter_and_result_profile'
		   if ($4.atributs.tipus == TYPE) {
				// Tipus bàsic
				info_function->tipus_retorn = $4.atributs.continguts.info_type.tipus;
				if ($4.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT) {
					memcpy(info_function->nom_tipus_retorn, $4.nom, STRING_LENGTH);
				}
		   } else if ($4.atributs.tipus == ENUMERATED_TYPE) {
				// A partir del nom podem extreure el simbol del tipus
				info_function->tipus_retorn = TIPUS_DERIVAT;
				memcpy(info_function->nom_tipus_retorn, $4.nom, STRING_LENGTH);
		   } else if ($4.atributs.tipus == RECORD_TYPE) {
				info_function->tipus_retorn = TIPUS_DERIVAT;
				memcpy(info_function->nom_tipus_retorn, $4.nom, STRING_LENGTH);
		   } else if ($4.atributs.tipus == ARRAY_TYPE) {
				info_function->tipus_retorn = TIPUS_DERIVAT;
				if (strcmp($4.nom, "") == 0) {
					// NOTE: Considerem que no poden haver-hi tipus anonims a la signatura.
					sinError("No poden haver-hi tipus anonims a la signatura.");
				} else {
					// Array ya definit
					memcpy(info_function->nom_tipus_retorn, $4.nom, STRING_LENGTH);
				}
		   } else {
				sinError("Sense tipus vàlid de retorn! %i", $4.atributs.tipus);
		   }
		   
		   encuar($2->atributs.continguts.info_function.sobrecarregues, info_function);
		   
		   // Util per quan es troba un RETURN al cos, es pugui marcar el simbol de la funció a TRUE
		   infoFunctionActual = info_function;
		   
		   // Retornem el simbol original amb els info_function de la sobrecarrega nova
		   memcpy(&$$, $2, sizeof(struct Simbol));
		   memcpy(&($$.atributs.continguts.info_function), info_function , sizeof(struct Info_Function));
	   } else {
		   // Hem trobat la sobrecarrega ya definida, no cal definir-la de nou
		   // TODO: Alliberar taula_tmp i cua_strings_tmp
		   // Retornem el simbol original amb els Info_Function de la sobrecarrega trobada
		   memcpy(&$$, $2, sizeof(struct Simbol));
		   memcpy(&($$.atributs.continguts.info_function), info_function_sobrecarregat , sizeof(struct Info_Function));
	   }
   }
;

defining_designator:
   defining_program_unit_name {
	    sinLog(" -> defining_designator -> defining_program_unit_name");
		// Pasem el nom de l'identificador cap adalt
		snprintf($$, STRING_LENGTH, "%s" , $1);
   }
;

parameter_profile:
   formal_part {sinLog(" -> parameter_profile -> formal_part");}
   | {sinLog(" -> parameter_profile");}
;

parameter_and_result_profile:
   formal_part PR_RETURN subtype_mark {
	   sinLog("parameter_and_result_profile -> formal_part PR_RETURN subtype_mark");
	   memcpy(&$$, &$3, sizeof(struct Simbol));
   }
   | PR_RETURN subtype_mark {
	   sinLog("parameter_and_result_profile -> PR_RETURN subtype_mark");
	   memcpy(&$$, &$2, sizeof(struct Simbol));
   }
;

formal_part:
   '(' parameter_specification formal_part_element {sinLog(" -> formal_part -> '(' parameter_specification formal_part_element");}
;

formal_part_element:
   ';' parameter_specification formal_part_element { sinLog(" -> formal_part_element -> ';' parameter_specification formal_part_element"); }
   | ';' parameter_specification {sinLog(" -> formal_part_element -> parameter_specification");}
   | ')' {sinLog(" -> formal_part_element -> ')'");}
;

parameter_specification:
   defining_identifier_list ':' subtype_mark {
	    // Treure de la cua i ficar a la TS temporal
		reiniciar_PDI(cua_defining_identifier_tmp); // Cua inicialitzada i muntada a defining_identifier_list
		Simbol *element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		while (element != NULL) {
			if ($3.atributs.tipus == TYPE) {
				// Tipus bàsic
				element->atributs.continguts.info_variable.tipus = $3.atributs.continguts.info_type.tipus;
				if ($3.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT) {
					memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
				}
			} else if ($3.atributs.tipus == ENUMERATED_TYPE) {
				// A partir del nom podem extreure el simbol del tipus
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
			} else if ($3.atributs.tipus == RECORD_TYPE) {
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
			} else if ($3.atributs.tipus == ARRAY_TYPE) {
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				if (strcmp($3.nom, "") == 0) {
					// NOTE: Considerem que no poden haver-hi tipus anonims a la signatura.
					sinError("No poden haver-hi tipus anonims a la signatura.");
					/*// Array definit in situ (TIPUS ANONIM)
					
					// Afegim array definit in situ a la TS com si fos un tipus array definit amb el nom "$" + "nom_variable"
					Simbol *symTipus = (Simbol*)malloc(sizeof(struct Simbol));
					sprintf(symTipus->nom, "$%s", element->nom);
					memcpy(&(symTipus->atributs), &($3.atributs), sizeof(Atributs));
					inserir_simbol_en_taula(taula_tmp, symTipus);
					
					// Associem la variable al tipus anterior
					memcpy(element->atributs.continguts.info_variable.nom_tipus, symTipus->nom, STRING_LENGTH);*/
				} else {
					// Array ya definit
					memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
				}
			} else {
				sinError("Sense tipus vàlid! %i", $3.atributs.tipus);
			}
			
			// Encuem el nom del paràmetre per tindre un llistat de parametres de procediment/funcio
			encuar(cua_strings_tmp, element->nom);
			num_elements_cua_strings_tmp++;
			
			inserir_simbol_en_taula(taula_tmp, element);
			seguent_PDI(cua_defining_identifier_tmp);
			element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		}
		cua_defining_identifier_tmp = NULL;
   }
;

object_declaration:
   defining_identifier_list ':' PR_CONSTANT subtype_indication PR_EQU expression ';' {
	    // Treure de la cua i ficar a la TS temporal
		reiniciar_PDI(cua_defining_identifier_tmp);	// Cua inicialitzada i muntada a defining_identifier_list
		Simbol *element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		while (element != NULL) {
			element->atributs.continguts.info_variable.constant = TRUE;
			element->atributs.continguts.info_variable.inicialitzat = TRUE;
			
			if ($4.atributs.tipus == TYPE) {
				// Tipus bàsic
				element->atributs.continguts.info_variable.tipus = $4.atributs.continguts.info_type.tipus;
				
				// La expressió de $5 mai sera TIPUS_INTEGER_DERIVAT, s'accepta si la variable es integer derivat i
				// es comprova els rangs més tard.
				if (!(($6.atributs.continguts.info_type.tipus == TIPUS_INTEGER)
					&& ($4.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT))) {
				
					if ($4.atributs.continguts.info_type.tipus != $6.atributs.continguts.info_variable.tipus) {
						sinError("La variable no es del mateix tipus que el valor que s'intenta assignar.");
					}
				}
				
				
				if ($6.atributs.continguts.info_variable.literal == TRUE) {
					switch ($4.atributs.continguts.info_type.tipus) {
						// NOTE: La comprovació de rangs es fa al lexic excepte pels INTEGER_DERIVAT
						case TIPUS_INTEGER:
							element->atributs.continguts.info_variable.valor.integer = $6.atributs.continguts.info_variable.valor.integer;
							break;
						case TIPUS_INTEGER_DERIVAT:
							if (($6.atributs.continguts.info_variable.valor.integer < (int)$4.atributs.continguts.info_type.rang.min)
									|| ($6.atributs.continguts.info_variable.valor.integer > (int)$4.atributs.continguts.info_type.rang.max)) {
								sinError("Valor fora de rang, ha d'estar entre %i i %i.", (int)$4.atributs.continguts.info_type.rang.min, (int)$4.atributs.continguts.info_type.rang.max);
							}
							if (strcmp($4.nom, "") == 0) {
								// Subtipus integer definit in situ (TIPUS ANONIM)
					
								// Afegim subtipus integer definit in situ a la TS com si fos un tipus array definit amb el nom "$" + "nom_variable"
								Simbol *symTipus = (Simbol*)malloc(sizeof(struct Simbol));
								sprintf(symTipus->nom, "$%s", element->nom);
								memcpy(&(symTipus->atributs), &($4.atributs), sizeof(Atributs));
								inserir_simbol_en_taula(taulaSimbolActual, symTipus);
								
								// Associem la variable al tipus anterior
								memcpy(element->atributs.continguts.info_variable.nom_tipus, symTipus->nom, STRING_LENGTH);
							} else {
								// Subtipus integer ja definit
								memcpy(element->atributs.continguts.info_variable.nom_tipus, $4.nom, STRING_LENGTH);
							}
							element->atributs.continguts.info_variable.valor.integer = $6.atributs.continguts.info_variable.valor.integer;
							break;
						case TIPUS_FLOAT:
							element->atributs.continguts.info_variable.valor.vfloat = $6.atributs.continguts.info_variable.valor.vfloat;
							break;
						case TIPUS_BOOLEAN:
							element->atributs.continguts.info_variable.valor.boolean = $6.atributs.continguts.info_variable.valor.boolean;
							break;
						case TIPUS_CHARACTER:
							element->atributs.continguts.info_variable.valor.character = $6.atributs.continguts.info_variable.valor.character;
							break;
						case TIPUS_STRING:
							snprintf(element->atributs.continguts.info_variable.valor.string, CONTENIDOR_STRING_LENGTH, "%s", $6.atributs.continguts.info_variable.valor.string);
							break;
						case TIPUS_DERIVAT:
							// Es possible trobar-se en aquest punt? Crec que no.
							sinError("Comprovar error.");
							break;
					}
				}
			} else if ($4.atributs.tipus == ENUMERATED_TYPE) {
				// A partir del nom podem extreure el simbol del tipus
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				memcpy(element->atributs.continguts.info_variable.nom_tipus, $4.nom, STRING_LENGTH);
				
				Simbol *symEnumElement = busca_totes_taules(taulaSimbolActual, $6.nom);
				if (symEnumElement == NULL) {
					sinError("El valor que s'intenta assignar no existeix com enumerador.");
				}
				
				if (strcmp($4.nom, symEnumElement->atributs.continguts.info_variable.nom_tipus) != 0) {
					sinError("El valor que s'intenta assignar no pertany al mateix enumerat que defineix el tipus de la variable.");
				}
				
				// NOTE: A una variable del tipus enumerat guardem a 'integer' el valor correponent al element del enumerat assignat
				element->atributs.continguts.info_variable.valor.integer = symEnumElement->atributs.continguts.info_variable.valor.integer;
				
			} else if ($4.atributs.tipus == RECORD_TYPE) {
				// NOTE: No permetem inicialització de RECORDs a la declaracio
				sinError("No es pot inicialitzar un tipus RECORD a la seva declaració.");
			} else if ($4.atributs.tipus == ARRAY_TYPE) {
				// NOTE: No permetem inicialització de ARRAY a la declaracio
				sinError("No es pot inicialitzar un tipus ARRAY a la seva declaració.");
			} else {
				sinError("Sense tipus vàlid! %i", $4.atributs.tipus);
			}
			
			inserir_simbol_en_taula(taulaSimbolActual, element);
			seguent_PDI(cua_defining_identifier_tmp);
			element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		}
		cua_defining_identifier_tmp = NULL;
   }
   | defined_identifier ':' PR_CONSTANT subtype_indication PR_EQU expression ';' {
	   // Controlar errors de variables definides amb mateix nom
	   // Saltarà l'error a defined_identifier
   }
   | defining_identifier_list ':' subtype_indication PR_EQU expression ';' {
	   // Treure de la cua i ficar a la TS temporal
		reiniciar_PDI(cua_defining_identifier_tmp);	// Cua inicialitzada i muntada a defining_identifier_list
		Simbol *element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		while (element != NULL) {
			element->atributs.continguts.info_variable.constant = FALSE;
			element->atributs.continguts.info_variable.inicialitzat = TRUE;
			
			if ($3.atributs.tipus == TYPE) {
				// Tipus bàsic
				element->atributs.continguts.info_variable.tipus = $3.atributs.continguts.info_type.tipus;
				
				// La expressió de $5 mai sera TIPUS_INTEGER_DERIVAT, s'accepta si la variable es integer derivat i
				// es comprova els rangs més tard.
				if (!(($5.atributs.continguts.info_type.tipus == TIPUS_INTEGER)
					&& ($3.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT))) {
				
					if ($3.atributs.continguts.info_type.tipus != $5.atributs.continguts.info_variable.tipus) {
						sinError("La variable no es del mateix tipus que el valor que s'intenta assignar.");
					}
				}
				
				if ($5.atributs.continguts.info_variable.literal == TRUE) {
					switch ($3.atributs.continguts.info_type.tipus) {
						// NOTE: La comprovació de rangs es fa al lexic excepte pels INTEGER_DERIVAT
						case TIPUS_INTEGER:
							element->atributs.continguts.info_variable.valor.integer = $5.atributs.continguts.info_variable.valor.integer;
							break;
						case TIPUS_INTEGER_DERIVAT:
							if (($5.atributs.continguts.info_variable.valor.integer < (int)$3.atributs.continguts.info_type.rang.min)
									|| ($5.atributs.continguts.info_variable.valor.integer > (int)$3.atributs.continguts.info_type.rang.max)) {
								sinError("Valor fora de rang, ha d'estar entre %i i %i.", (int)$3.atributs.continguts.info_type.rang.min, (int)$3.atributs.continguts.info_type.rang.max);
							}
							if (strcmp($3.nom, "") == 0) {
								// Subtipus integer definit in situ (TIPUS ANONIM)
					
								// Afegim subtipus integer definit in situ a la TS com si fos un tipus array definit amb el nom "$" + "nom_variable"
								Simbol *symTipus = (Simbol*)malloc(sizeof(struct Simbol));
								sprintf(symTipus->nom, "$%s", element->nom);
								memcpy(&(symTipus->atributs), &($3.atributs), sizeof(Atributs));
								inserir_simbol_en_taula(taulaSimbolActual, symTipus);
								
								// Associem la variable al tipus anterior
								memcpy(element->atributs.continguts.info_variable.nom_tipus, symTipus->nom, STRING_LENGTH);
							} else {
								// Subtipus integer ja definit
								memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
							}
							element->atributs.continguts.info_variable.valor.integer = $5.atributs.continguts.info_variable.valor.integer;
							break;
						case TIPUS_FLOAT:
							element->atributs.continguts.info_variable.valor.vfloat = $5.atributs.continguts.info_variable.valor.vfloat;
							break;
						case TIPUS_BOOLEAN:
							element->atributs.continguts.info_variable.valor.boolean = $5.atributs.continguts.info_variable.valor.boolean;
							break;
						case TIPUS_CHARACTER:
							element->atributs.continguts.info_variable.valor.character = $5.atributs.continguts.info_variable.valor.character;
							break;
						case TIPUS_STRING:
							snprintf(element->atributs.continguts.info_variable.valor.string, CONTENIDOR_STRING_LENGTH, "%s", $5.atributs.continguts.info_variable.valor.string);
							break;
						case TIPUS_DERIVAT:
							// Es possible trobar-se en aquest punt? Crec que no.
							sinError("Comprovar error.");
							break;
					}
				}
			} else if ($3.atributs.tipus == ENUMERATED_TYPE) {
				// A partir del nom podem extreure el simbol del tipus
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
				
				Simbol *symEnumElement = busca_totes_taules(taulaSimbolActual, $5.nom);
				if (symEnumElement == NULL) {
					sinError("El valor que s'intenta assignar no existeix com enumerador.");
				}
				
				if (strcmp($3.nom, symEnumElement->atributs.continguts.info_variable.nom_tipus) != 0) {
					sinError("El valor que s'intenta assignar no pertany al mateix enumerat que defineix el tipus de la variable.");
				}
				
				// NOTE: A una variable del tipus enumerat guardem a 'integer' el valor correponent al element del enumerat assignat
				element->atributs.continguts.info_variable.valor.integer = symEnumElement->atributs.continguts.info_variable.valor.integer;
				
			} else if ($3.atributs.tipus == RECORD_TYPE) {
				// NOTE: No permetem inicialització de RECORDs a la declaracio
				sinError("No es pot inicialitzar un tipus RECORD a la seva declaració.");
			} else if ($3.atributs.tipus == ARRAY_TYPE) {
				// NOTE: No permetem inicialització de ARRAY a la declaracio
				sinError("No es pot inicialitzar un tipus ARRAY a la seva declaració.");
			} else {
				sinError("Sense tipus vàlid! %i", $3.atributs.tipus);
			}
			
			inserir_simbol_en_taula(taulaSimbolActual, element);
			seguent_PDI(cua_defining_identifier_tmp);
			element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		}
		cua_defining_identifier_tmp = NULL;
   }
   | defined_identifier ':' subtype_indication PR_EQU expression ';' {
	   // Controlar errors de variables definides amb mateix nom
	   // Saltarà l'error a defined_identifier
   }
   | defining_identifier_list ':' PR_CONSTANT subtype_indication ';' {
	   // Treure de la cua i ficar a la TS temporal
		reiniciar_PDI(cua_defining_identifier_tmp);	// Cua inicialitzada i muntada a defining_identifier_list
		Simbol *element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		while (element != NULL) {
			element->atributs.continguts.info_variable.constant = TRUE;
			element->atributs.continguts.info_variable.inicialitzat = FALSE;
			
			if ($4.atributs.tipus == TYPE) {
				// Tipus bàsic
				element->atributs.continguts.info_variable.tipus = $4.atributs.continguts.info_type.tipus;
				if ($4.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT) {
					if (strcmp($4.nom, "") == 0) {
						// Subtipus integer definit in situ (TIPUS ANONIM)
			
						// Afegim subtipus integer definit in situ a la TS com si fos un tipus array definit amb el nom "$" + "nom_variable"
						Simbol *symTipus = (Simbol*)malloc(sizeof(struct Simbol));
						sprintf(symTipus->nom, "$%s", element->nom);
						memcpy(&(symTipus->atributs), &($4.atributs), sizeof(Atributs));
						inserir_simbol_en_taula(taulaSimbolActual, symTipus);
						
						// Associem la variable al tipus anterior
						memcpy(element->atributs.continguts.info_variable.nom_tipus, symTipus->nom, STRING_LENGTH);
					} else {
						// Subtipus integer ja definit
						memcpy(element->atributs.continguts.info_variable.nom_tipus, $4.nom, STRING_LENGTH);
					}
				}
			} else if ($4.atributs.tipus == ENUMERATED_TYPE) {
				// A partir del nom podem extreure el simbol del tipus
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				memcpy(element->atributs.continguts.info_variable.nom_tipus, $4.nom, STRING_LENGTH);
			} else if ($4.atributs.tipus == RECORD_TYPE) {
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				memcpy(element->atributs.continguts.info_variable.nom_tipus, $4.nom, STRING_LENGTH);
				// Dupliquem la taula de simbols del RECORD, donat que aquesta conte les variables del RECORD
				// i les podrem utilitzar per guardar valors.
				element->atributs.continguts.info_variable.valor.taula = intentar_duplicar_taula_simbols($4.atributs.continguts.info_record.elements);
				
				if (element->atributs.continguts.info_variable.valor.taula == NULL) {
					// NOTE: Comprovar que funciona.
					sinError("Imposible duplicar taula!");
				}
			} else if ($4.atributs.tipus == ARRAY_TYPE) {
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				// Arrays anonims tractats a una altra branca, aquí sempre es troben ja definits
				memcpy(element->atributs.continguts.info_variable.nom_tipus, $4.nom, STRING_LENGTH);
			} else {
				sinError("Sense tipus vàlid! %i", $4.atributs.tipus);
			}
			
			inserir_simbol_en_taula(taulaSimbolActual, element);
			seguent_PDI(cua_defining_identifier_tmp);
			element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		}
		cua_defining_identifier_tmp = NULL;
   }
   | defined_identifier ':' PR_CONSTANT subtype_indication ';' {
	   // Controlar errors de variables definides amb mateix nom
	   // Saltarà l'error a defined_identifier
   }
   | defining_identifier_list ':' subtype_indication ';' {
	   // Treure de la cua i ficar a la TS temporal
		reiniciar_PDI(cua_defining_identifier_tmp);	// Cua inicialitzada i muntada a defining_identifier_list
		Simbol *element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		while (element != NULL) {
			element->atributs.continguts.info_variable.constant = FALSE;
			element->atributs.continguts.info_variable.inicialitzat = FALSE;
			
			if ($3.atributs.tipus == TYPE) {
				// Tipus bàsic
				element->atributs.continguts.info_variable.tipus = $3.atributs.continguts.info_type.tipus;
				if ($3.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT) {
					if (strcmp($3.nom, "") == 0) {
						// Subtipus integer definit in situ (TIPUS ANONIM)
			
						// Afegim subtipus integer definit in situ a la TS com si fos un tipus array definit amb el nom "$" + "nom_variable"
						Simbol *symTipus = (Simbol*)malloc(sizeof(struct Simbol));
						sprintf(symTipus->nom, "$%s", element->nom);
						memcpy(&(symTipus->atributs), &($3.atributs), sizeof(Atributs));
						inserir_simbol_en_taula(taulaSimbolActual, symTipus);
						
						// Associem la variable al tipus anterior
						memcpy(element->atributs.continguts.info_variable.nom_tipus, symTipus->nom, STRING_LENGTH);
					} else {
						// Subtipus integer ja definit
						memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
					}
				}
			} else if ($3.atributs.tipus == ENUMERATED_TYPE) {
				// A partir del nom podem extreure el simbol del tipus
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
			} else if ($3.atributs.tipus == RECORD_TYPE) {
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
				// Dupliquem la taula de simbols del RECORD, donat que aquesta conte les variables del RECORD
				// i les podrem utilitzar per guardar valors.
				element->atributs.continguts.info_variable.valor.taula = intentar_duplicar_taula_simbols($3.atributs.continguts.info_record.elements);
				
				if (element->atributs.continguts.info_variable.valor.taula == NULL) {
					// NOTE: Comprovar que funciona.
					sinError("Imposible duplicar taula!");
				}
			} else if ($3.atributs.tipus == ARRAY_TYPE) {
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				// Arrays anonims tractats a una altra branca, aquí sempre es troben ja definits
				memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
			} else {
				sinError("Sense tipus vàlid! %i", $3.atributs.tipus);
			}
			
			inserir_simbol_en_taula(taulaSimbolActual, element);
			seguent_PDI(cua_defining_identifier_tmp);
			element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		}
		cua_defining_identifier_tmp = NULL;
   }
   | defined_identifier ':' subtype_indication ';' {
	   // Controlar errors de variables definides amb mateix nom
	   // Saltarà l'error a defined_identifier
   }
   | defining_identifier_list ':' PR_CONSTANT array_type_definition PR_EQU expression ';' {
	   // ARRAY anonim
	   // Treure de la cua i ficar a la TS temporal
		reiniciar_PDI(cua_defining_identifier_tmp);	// Cua inicialitzada i muntada a defining_identifier_list
		Simbol *element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		while (element != NULL) {
			element->atributs.continguts.info_variable.constant = TRUE;
			element->atributs.continguts.info_variable.inicialitzat = FALSE;
			
			if ($4.atributs.tipus == ARRAY_TYPE) {
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				// Array definit in situ (TIPUS ANONIM)
				// $3.nom sempre ha de ser igual a "" en aquest cas
					
				// Afegim array definit in situ a la TS com si fos un tipus array definit amb el nom "$" + "nom_variable"
				Simbol *symTipus = (Simbol*)malloc(sizeof(struct Simbol));
				sprintf(symTipus->nom, "$%s", element->nom);
				memcpy(&(symTipus->atributs), &($4.atributs), sizeof(Atributs));
				inserir_simbol_en_taula(taulaSimbolActual, symTipus);
				
				// Associem la variable al tipus anterior
				memcpy(element->atributs.continguts.info_variable.nom_tipus, symTipus->nom, STRING_LENGTH);
			} else {
				sinError("Només s'espera un tipus array anonim aqui.");
			}
			
			inserir_simbol_en_taula(taulaSimbolActual, element);
			seguent_PDI(cua_defining_identifier_tmp);
			element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		}
		cua_defining_identifier_tmp = NULL;
		
		
		if ($4.atributs.continguts.info_array.tipus != $6.atributs.continguts.info_variable.tipus) {
			sinError("El tipus assignat no és del mateix tipus que l'array.");
		}
		
		// Assignació de valors
		switch ($6.atributs.continguts.info_array.tipus) {
			// TODO: Assignar valor a totes les posicions del array
			case TIPUS_INTEGER:
				// Buscar integer a la taula de simbols i comprovar rangs
				//element->atributs.continguts.info_variable.valor.integer = $5.atributs.continguts.info_variable.valor.integer;
				break;
			case TIPUS_INTEGER_DERIVAT:
				// Buscar subtipus integer a la taula de simbols i comprovar rangs
				/*if (($6.atributs.continguts.info_variable.valor.integer < $4.atributs.continguts.info_type.rang.min)
						|| ($6.atributs.continguts.info_variable.valor.integer > $4.atributs.continguts.info_type.rang.max)) {
					sinError("Valor fora de rang, ha d'estar entre %i i %i.", $4.atributs.continguts.info_type.rang.min, $4.atributs.continguts.info_type.rang.max);
				}*/
				//element->atributs.continguts.info_variable.valor.integer = $5.atributs.continguts.info_variable.valor.integer;
				break;
			case TIPUS_FLOAT:
				//element->atributs.continguts.info_variable.valor.vfloat = $6.atributs.continguts.info_variable.valor.vfloat;
				break;
			case TIPUS_BOOLEAN:
				//element->atributs.continguts.info_variable.valor.boolean = $6.atributs.continguts.info_variable.valor.boolean;
				break;
			case TIPUS_CHARACTER:
				//element->atributs.continguts.info_variable.valor.character = $6.atributs.continguts.info_variable.valor.character;
				break;
			case TIPUS_STRING:
				//snprintf(element->atributs.continguts.info_variable.valor.string, STRING_LENGTH, "%s", $6.atributs.continguts.info_variable.valor.string);
				break;
			case TIPUS_DERIVAT:
				// NOTE: No es permeten arrays de tipus que no siguin bàsics (integer, character, float, boolean i string).
				sinError("No es permeten arrays de tipus que no siguin bàsics (integer, character, float, boolean i string).");
				break;
		}
		
   }
   | defined_identifier ':' PR_CONSTANT array_type_definition PR_EQU expression ';' {
	   // Controlar errors de variables definides amb mateix nom
	   // Saltarà l'error a defined_identifier
   }
   | defining_identifier_list ':' array_type_definition PR_EQU expression ';' {
	   // ARRAY anonim
	   // Treure de la cua i ficar a la TS temporal
		reiniciar_PDI(cua_defining_identifier_tmp);	// Cua inicialitzada i muntada a defining_identifier_list
		Simbol *element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		while (element != NULL) {
			element->atributs.continguts.info_variable.constant = FALSE;
			element->atributs.continguts.info_variable.inicialitzat = FALSE;
			
			if ($3.atributs.tipus == ARRAY_TYPE) {
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				// Array definit in situ (TIPUS ANONIM)
				// $3.nom sempre ha de ser igual a "" en aquest cas
					
				// Afegim array definit in situ a la TS com si fos un tipus array definit amb el nom "$" + "nom_variable"
				Simbol *symTipus = (Simbol*)malloc(sizeof(struct Simbol));
				sprintf(symTipus->nom, "$%s", element->nom);
				memcpy(&(symTipus->atributs), &($3.atributs), sizeof(Atributs));
				inserir_simbol_en_taula(taulaSimbolActual, symTipus);
				
				// Associem la variable al tipus anterior
				memcpy(element->atributs.continguts.info_variable.nom_tipus, symTipus->nom, STRING_LENGTH);
			} else {
				sinError("Només s'espera un tipus array anonim aqui.");
			}
			
			inserir_simbol_en_taula(taulaSimbolActual, element);
			seguent_PDI(cua_defining_identifier_tmp);
			element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		}
		cua_defining_identifier_tmp = NULL;
		
		if ($3.atributs.continguts.info_array.tipus != $5.atributs.continguts.info_variable.tipus) {
			sinError("El tipus assignat no és del mateix tipus que l'array.");
		}
		
		// Assignació de valors
		switch ($5.atributs.continguts.info_array.tipus) {
			// TODO: Assignar valor a totes les posicions del array
			case TIPUS_INTEGER:
				// Buscar integer a la taula de simbols i comprovar rangs
				//element->atributs.continguts.info_variable.valor.integer = $5.atributs.continguts.info_variable.valor.integer;
				break;
			case TIPUS_INTEGER_DERIVAT:
				// Buscar subtipus integer a la taula de simbols i comprovar rangs
				/*if (($5.atributs.continguts.info_variable.valor.integer < $3.atributs.continguts.info_type.rang.min)
						|| ($5.atributs.continguts.info_variable.valor.integer > $3.atributs.continguts.info_type.rang.max)) {
					sinError("Valor fora de rang, ha d'estar entre %i i %i.", $3.atributs.continguts.info_type.rang.min, $3.atributs.continguts.info_type.rang.max);
				}*/
				//element->atributs.continguts.info_variable.valor.integer = $5.atributs.continguts.info_variable.valor.integer;
				break;
			case TIPUS_FLOAT:
				//element->atributs.continguts.info_variable.valor.vfloat = $5.atributs.continguts.info_variable.valor.vfloat;
				break;
			case TIPUS_BOOLEAN:
				//element->atributs.continguts.info_variable.valor.boolean = $5.atributs.continguts.info_variable.valor.boolean;
				break;
			case TIPUS_CHARACTER:
				//element->atributs.continguts.info_variable.valor.character = $5.atributs.continguts.info_variable.valor.character;
				break;
			case TIPUS_STRING:
				//snprintf(element->atributs.continguts.info_variable.valor.string, STRING_LENGTH, "%s", $5.atributs.continguts.info_variable.valor.string);
				break;
			case TIPUS_DERIVAT:
				// NOTE: No es permeten arrays de tipus que no siguin bàsics (integer, character, float, boolean i string).
				sinError("No es permeten arrays de tipus que no siguin bàsics (integer, character, float, boolean i string).");
				break;
		}
   }
   | defined_identifier ':' array_type_definition PR_EQU expression ';' {
	   // Controlar errors de variables definides amb mateix nom
	   // Saltarà l'error a defined_identifier
   }
   | defining_identifier_list ':' PR_CONSTANT array_type_definition ';' {
	   // ARRAY anonim
	   // Treure de la cua i ficar a la TS temporal
		reiniciar_PDI(cua_defining_identifier_tmp);	// Cua inicialitzada i muntada a defining_identifier_list
		Simbol *element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		while (element != NULL) {
			element->atributs.continguts.info_variable.constant = TRUE;
			element->atributs.continguts.info_variable.inicialitzat = FALSE;
			
			if ($4.atributs.tipus == ARRAY_TYPE) {
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				// Array definit in situ (TIPUS ANONIM)
				// $3.nom sempre ha de ser igual a "" en aquest cas
					
				// Afegim array definit in situ a la TS com si fos un tipus array definit amb el nom "$" + "nom_variable"
				Simbol *symTipus = (Simbol*)malloc(sizeof(struct Simbol));
				sprintf(symTipus->nom, "$%s", element->nom);
				memcpy(&(symTipus->atributs), &($4.atributs), sizeof(Atributs));
				inserir_simbol_en_taula(taulaSimbolActual, symTipus);
				
				// Associem la variable al tipus anterior
				memcpy(element->atributs.continguts.info_variable.nom_tipus, symTipus->nom, STRING_LENGTH);
			} else {
				sinError("Només s'espera un tipus array anonim aqui.");
			}
			
			inserir_simbol_en_taula(taulaSimbolActual, element);
			seguent_PDI(cua_defining_identifier_tmp);
			element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		}
		cua_defining_identifier_tmp = NULL;
   }
   | defined_identifier ':' PR_CONSTANT array_type_definition ';' {
	   // Controlar errors de variables definides amb mateix nom
	   // Saltarà l'error a defined_identifier
   }
   | defining_identifier_list ':' array_type_definition ';' {
	    // ARRAY anonim
	    // Treure de la cua i ficar a la TS temporal
		reiniciar_PDI(cua_defining_identifier_tmp);	// Cua inicialitzada i muntada a defining_identifier_list
		Simbol *element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		while (element != NULL) {
			element->atributs.continguts.info_variable.constant = FALSE;
			element->atributs.continguts.info_variable.inicialitzat = FALSE;
			
			if ($3.atributs.tipus == ARRAY_TYPE) {
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				// Array definit in situ (TIPUS ANONIM)
				// $3.nom sempre ha de ser igual a "" en aquest cas
					
				// Afegim array definit in situ a la TS com si fos un tipus array definit amb el nom "$" + "nom_variable"
				Simbol *symTipus = (Simbol*)malloc(sizeof(struct Simbol));
				sprintf(symTipus->nom, "$%s", element->nom);
				memcpy(&(symTipus->atributs), &($3.atributs), sizeof(Atributs));
				inserir_simbol_en_taula(taulaSimbolActual, symTipus);
				
				// Associem la variable al tipus anterior
				memcpy(element->atributs.continguts.info_variable.nom_tipus, symTipus->nom, STRING_LENGTH);
			} else {
				sinError("Només s'espera un tipus array anonim aqui.");
			}
			
			inserir_simbol_en_taula(taulaSimbolActual, element);
			seguent_PDI(cua_defining_identifier_tmp);
			element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		}
		cua_defining_identifier_tmp = NULL;
   }
   | defined_identifier ':' array_type_definition ';' {
	   // Controlar errors de variables definides amb mateix nom
	   // Saltarà l'error a defined_identifier
   }
;

type_declaration:
   full_type_declaration {sinLog(" -> type_declaration -> full_type_declaration");}
;

full_type_declaration:
	// NOTE: Segons l'enunciat no sembla que sagi de soportar els tipus incomplets
   PR_TYPE defining_identifier PR_IS type_definition ';' {
	   Simbol *sym, *sym_tmp;
	   int i;
	   
	   sinLog(" -> full_type_declaration -> PR_TYPE defining_identifier PR_IS type_definition ';' ");

	   sinLog("El tipus complet \"%s\" s'ha insertat a la taula de simbols.", $2);
	   sym = crear_i_inserir_en_taula(taulaSimbolActual, $2);
	   //sym->atributs.continguts.info_type.tipus = TIPUS_BOOLEAN;
	   sym->atributs.tipus = $4.atributs.tipus;
	   
	   if (sym->atributs.tipus == ENUMERATED_TYPE) {
			sym->atributs.continguts.info_enumerated.elements = $4.atributs.continguts.info_enumerated.elements;
			
			Cua *elements = sym->atributs.continguts.info_enumerated.elements;
			reiniciar_PDI(elements);
			Simbol *element = (Simbol *)obtenir_element_PDI(elements);
			int posicio = 0;
			while(element != NULL) {
				if (posicio > 255) {
					// NOTE: Segons l'enunciat un enumerat es de 8 bits, no poden haver més de 256 elements
					sinError("Un enumerat no pot tindre més de 256 elements.");
				}
				// Cada element del enumerat s'inserta com un simbol de tipus variable a la TS
				sym_tmp = crear_i_inserir_en_taula(taulaSimbolActual, element -> nom);
				memcpy(&(sym_tmp->atributs), &(element->atributs), sizeof(Atributs));
				sym_tmp->atributs.continguts.info_variable.tipus = TIPUS_INTEGER_DERIVAT;
				sym_tmp->atributs.continguts.info_variable.valor.integer = posicio;
				// Es lliga al tipus enumerat:
				sprintf(sym_tmp->atributs.continguts.info_variable.nom_tipus, "%s", $2);
				posicio++;
				seguent_PDI(elements);
				element = (Simbol *)obtenir_element_PDI(elements);
			}
			
		}
		
		if (sym->atributs.tipus == TYPE) {
			// Tipus 'integer' amb un rang redefinit
			sym->atributs.continguts.info_type.tipus = $4.atributs.continguts.info_type.tipus;
			sym->atributs.continguts.info_type.rang.min = $4.atributs.continguts.info_type.rang.min;
			sym->atributs.continguts.info_type.rang.max = $4.atributs.continguts.info_type.rang.max;
			sym->atributs.continguts.info_type.rang.tipus = $4.atributs.continguts.info_type.rang.tipus;
		}
		
		if (sym->atributs.tipus == RECORD_TYPE) {
			sym->atributs.continguts.info_record.elements = $4.atributs.continguts.info_record.elements;
		}
		
		if (sym->atributs.tipus == ARRAY_TYPE) {
			memcpy(&(sym->atributs), &($4.atributs), sizeof(Atributs));
		}
		
	}
	
	| PR_TYPE defined_identifier PR_IS type_definition ';' {
		// Controlar errors de tipus definits amb el mateix nom
		// Saltarà l'error a defined_identifier
	}
;

type_definition:
   enumeration_type_definition { 
	   sinLog(" -> type_definition -> enumeration_type_definition");
	   $$.atributs.tipus = $1.atributs.tipus; // ENUMERATED_TYPE
	   $$.atributs.continguts.info_enumerated.elements = $1.atributs.continguts.info_enumerated.elements;
   } 
   | integer_type_definition {
	   sinLog(" -> type_definition -> integer_type_definition");
	   memcpy(&$$, &$1, sizeof(struct Simbol));
   } 
   | array_type_definition {
	   sinLog(" -> type_definition -> array_type_definition");
	   memcpy(&$$, &$1, sizeof(struct Simbol));
   } 
   | record_type_definition {
	   sinLog("record_type_definition -> type_definition");
	   $$.atributs.tipus = RECORD_TYPE;
	   $$.atributs.continguts.info_record.elements = $1.atributs.continguts.info_record.elements;
   }
;

record_type_definition:
   record_definition {}
;

record_definition:
   // Potser cal inicialitzar taula de simbols entre PR_RECORD i component_list
   PR_RECORD {
	   taula_tmp = (Taula_Simbols *)crear_taula_simbols("RECORD", taulaSimbolActual);
   }
   component_list PR_END PR_RECORD {
	   sinLog(" -> record_definition -> T_RECORD component_list T_END T_RECORD");
	   $$.atributs.continguts.info_record.elements = taula_tmp;
	   taula_tmp = NULL;
   }
;

component_list:
   component_item {}
   | component_item component_list {}
;

component_item:
   component_declaration {}
;

component_declaration:
	defining_identifier_list ':' component_definition ';' {
		// Treure de la cua i ficar a la TS temporal
		reiniciar_PDI(cua_defining_identifier_tmp); // Cua inicialitzada i muntada a defining_identifier_list
		Simbol *element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		while (element != NULL) {
			if ($3.atributs.tipus == TYPE) {
				// Tipus bàsic
				element->atributs.continguts.info_variable.tipus = $3.atributs.continguts.info_type.tipus;
				if ($3.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT) {
					
					if (strcmp($3.nom, "") == 0) {
						// Subtipus integer definit in situ (TIPUS ANONIM)
			
						// Afegim subtipus integer definit in situ a la TS com si fos un tipus array definit amb el nom "$" + "nom_variable"
						Simbol *symTipus = (Simbol*)malloc(sizeof(struct Simbol));
						sprintf(symTipus->nom, "$%s", element->nom);
						memcpy(&(symTipus->atributs), &($3.atributs), sizeof(Atributs));
						inserir_simbol_en_taula(taulaSimbolActual, symTipus);
						
						// Associem la variable al tipus anterior
						memcpy(element->atributs.continguts.info_variable.nom_tipus, symTipus->nom, STRING_LENGTH);
					} else {
						// Subtipus integer ja definit
						memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
					}
				}
				
			} else if ($3.atributs.tipus == ENUMERATED_TYPE) {
				// A partir del nom podem extreure el simbol del tipus
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
			} else if ($3.atributs.tipus == RECORD_TYPE) {
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
			} else if ($3.atributs.tipus == ARRAY_TYPE) {
				element->atributs.continguts.info_variable.tipus = TIPUS_DERIVAT;
				if (strcmp($3.nom, "") == 0) {
					// Array definit in situ (TIPUS ANONIM)
					
					// Afegim array definit in situ a la TS com si fos un tipus array definit amb el nom "$" + "nom_variable"
					Simbol *symTipus = (Simbol*)malloc(sizeof(struct Simbol));
					sprintf(symTipus->nom, "$%s", element->nom);
					memcpy(&(symTipus->atributs), &($3.atributs), sizeof(Atributs));
					inserir_simbol_en_taula(taulaSimbolActual, symTipus);
					
					// Associem la variable al tipus anterior
					memcpy(element->atributs.continguts.info_variable.nom_tipus, symTipus->nom, STRING_LENGTH);
				} else {
					// Array ya definit
					memcpy(element->atributs.continguts.info_variable.nom_tipus, $3.nom, STRING_LENGTH);
				}
			} else {
				sinError("Sense tipus vàlid! %i", $3.atributs.tipus);
			}
			
			inserir_simbol_en_taula(taula_tmp, element);
			seguent_PDI(cua_defining_identifier_tmp);
			element = (Simbol *)obtenir_element_PDI(cua_defining_identifier_tmp);
		}
		cua_defining_identifier_tmp = NULL;
	}
;

defining_identifier_list:
   // Insertem tots els identificadors del mateix tipus a una cua per després
   // (quan sapiguem de quin tipus son) insertar-los tots a la taula de simbols del record
   defining_identifier defining_identifier_element {
	   if (cua_defining_identifier_tmp == NULL) {
		   // RECORDATORI: Quan acabem d'utilitzar la cua, hem d'assignar-la a NULL
		   cua_defining_identifier_tmp = (Cua *)crear_cua();
	   }
	   sinLog(" -> defining_identifier_list -> defining_identifier defining_identifier_element");
	   if (cercar_simbol_a_cua(cua_defining_identifier_tmp, $1) == NULL) {
		   Simbol *sym = (Simbol*)malloc(sizeof(struct Simbol));
		   memcpy(sym->nom, $1, STRING_LENGTH); 
		   sym->atributs.tipus = VARIABLE;
		   sym->atributs.continguts.info_variable.valor.taula = NULL;
		   encuar(cua_defining_identifier_tmp, sym);
	   } else {
		   sinError("Redefinición del simbolo %s", $1);
	   }
   }
;

defining_identifier_element: /* Gramatica auxiliar de 'defining_identifier_list' */
   ',' defining_identifier defining_identifier_element {
	   if (cua_defining_identifier_tmp == NULL) {
		   // RECORDATORI: Quan acabem d'utilitzar la cua, hem d'assignar-la a NULL
		   cua_defining_identifier_tmp = (Cua *)crear_cua();
	   }
	   sinLog(" -> defining_identifier_element -> ',' defining_identifier defining_identifier_element");
	   if (cercar_simbol_a_cua(cua_defining_identifier_tmp, $2) == NULL) {
		   Simbol *sym = (Simbol*)malloc(sizeof(struct Simbol));
		   memcpy(sym->nom, $2, STRING_LENGTH); 
		   sym->atributs.tipus = VARIABLE;
		   sym->atributs.continguts.info_variable.valor.taula = NULL;
		   encuar(cua_defining_identifier_tmp, sym);
	   } else {
		   sinError("Redefinición del simbolo %s", $2);
	   }
   }
   | {}
;

array_type_definition:
   constrained_array_definition {
	    sinLog(" -> array_type_definition -> constrained_array_definition");
		memcpy(&$$, &$1, sizeof(struct Simbol));
   }
;

constrained_array_definition:
	// De discrete_subtype_definition ens arriba un ARRAY_TYPE
   PR_ARRAY '(' discrete_subtype_definition ')' PR_OF component_definition {
	   sinLog(" -> array_type_definition -> PR_ARRAY '(' discrete_subtype_definition ')' PR_OF component_definition");
	   memcpy(&$$, &$3, sizeof(struct Simbol));
	   
	   
	   $$.atributs.continguts.info_array.tipus = $6.atributs.tipus; // ARRAY_TYPE
	   if (($6.atributs.tipus == TYPE) && ($6.atributs.continguts.info_type.tipus == TIPUS_DERIVAT)) {
		   // NOTE: Comprovar que aquest cas no es pot donar
		   sinError("No pot ser TYPE i TIPUS_DERIVAT");
		   //snprintf($$.atributs.continguts.info_array.nom_tipus, STRING_LENGTH, "%s", $6.nom);
	   
	   } else if (($6.atributs.tipus == TYPE) && ($6.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT)) {
		   $$.atributs.continguts.info_array.tipus = $6.atributs.continguts.info_type.tipus;
		   snprintf($$.atributs.continguts.info_array.nom_tipus, STRING_LENGTH, "%s", $6.nom);
	   } else if ($6.atributs.tipus == TYPE) {
		   $$.atributs.continguts.info_array.tipus = $6.atributs.continguts.info_type.tipus;
	   } else if ($6.atributs.tipus == ENUMERATED_TYPE) {
	   		$$.atributs.continguts.info_array.tipus = TIPUS_DERIVAT;
		   snprintf($$.atributs.continguts.info_array.nom_tipus, STRING_LENGTH, "%s", $6.nom);
	   } else if (($6.atributs.tipus == RECORD_TYPE) || ($6.atributs.tipus == ARRAY_TYPE)) {
		   sinError("No es possible definir un array de tipus array o record");
	   }
   }
;

discrete_subtype_definition:
   subtype_indication {
	    sinLog(" -> discrete_subtype_definition -> subtype_indication");
		
		// A $1 hauriem de tindre TYPE, TIPUS_INTEGER.
		// Retornarem un ARRAY_TYPE
		if ($1.atributs.tipus == TYPE) {
			if (($1.atributs.continguts.info_type.tipus == TIPUS_INTEGER) || ($1.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT)) {
				// Agafem el rang del tipus
				sprintf($$.nom, "");	// No és encara un simbol de la TS
				$$.atributs.tipus = ARRAY_TYPE;
				$$.atributs.continguts.info_array.rang.min = $1.atributs.continguts.info_type.rang.min;
				$$.atributs.continguts.info_array.rang.max = $1.atributs.continguts.info_type.rang.max;
				$$.atributs.continguts.info_array.rang.tipus = $1.atributs.continguts.info_type.rang.tipus;
			} else {
				sinError("El tipus indicat no te cap rang definit (només es poden usar tipus 'integer' o derivats).");
			}
		} else {
			sinError("El tipus indicat no te cap rang definit (només es poden usar tipus 'integer' o derivats).");
		}
		
		
	} /*discrete_subtype_indication {}*/
   | range {
	    sinLog(" -> discrete_subtype_definition -> range");
		memcpy(&$$, &$1, sizeof(struct Simbol));
   }
;

subtype_indication:
	// Expected shift/reduce conflict
	subtype_mark {
		sinLog(" -> subtype_indication -> subtype_mark");
		memcpy(&$$, &$1, sizeof(struct Simbol));
	}
	| subtype_mark constraint {
		sinLog(" -> subtype_indication -> subtype_mark constraint");
		/*if ($1.fullType==FALSE)
			sinLog("The incomplete %s type can not be constrained",$1.name_type);*/
		
		memcpy(&$$, &$2, sizeof(struct Simbol)); // ARRAY_TYPE
		
		if ($1.atributs.tipus != TYPE) {
			sinError("Només es poden especificar subrangs a partir del tipus 'integer'.");
		}
		if ($1.atributs.continguts.info_type.tipus != TIPUS_INTEGER) {
			sinError("Només es poden especificar subrangs a partir del tipus 'integer'.");
		}
		
		// En 'contraint' ($2) ens arriba un ARRAY_TYPE
		sprintf($$.nom, "");	// No és encara un simbol de la TS
		$$.atributs.tipus = TYPE;
		$$.atributs.continguts.info_type.tipus = TIPUS_INTEGER_DERIVAT;
		$$.atributs.continguts.info_type.rang.min = $2.atributs.continguts.info_array.rang.min;
		$$.atributs.continguts.info_type.rang.max = $2.atributs.continguts.info_array.rang.min;
		$$.atributs.continguts.info_type.rang.tipus = TIPUS_INTEGER;
	}
;

subtype_mark:
   PR_ID_TYPE {
	   // El lexic haurà trobat el simbol corresponent a la TS i ens l'ha retornat a $1
	   // Passem cap adalt.
	   sinLog(" -> subtype_mark -> T_ID_TYPE"); 
	   memcpy(&$$, $1, sizeof(struct Simbol));
   }
   /*subtype_name {}*/
;

constraint:
   scalar_constraint {
	   sinLog(" -> constraint -> scalar_constraint");
	   memcpy(&$$, &$1, sizeof(struct Simbol)); // ARRAY_TYPE
   }
;

scalar_constraint:
   range_constraint {
	   sinLog(" -> scalar_constraint -> range_constraint");
	   memcpy(&$$, &$1, sizeof(struct Simbol)); // ARRAY_TYPE
   }
;

range_constraint:
   PR_RANGE range {
	    sinLog("-> range_constraint -> PR_RANGE range");
		memcpy(&$$, &$2, sizeof(struct Simbol)); // ARRAY_TYPE
   }
;

component_definition:
   subtype_indication {
	    sinLog(" -> component_definition -> subtype_indication");
		memcpy(&$$, &$1, sizeof(struct Simbol));
   }
;

integer_type_definition:
   signed_integer_type_definition {
	    sinLog(" -> integer_type_definition -> signed_integer_type_definition");
		memcpy(&$$, &$1, sizeof(struct Simbol));
   }
;

signed_integer_type_definition:
   PR_RANGE simple_expression PR_2_POINTS simple_expression {
   		sinLog(" -> signed_integer_type_definition -> PR_RANGE simple_expression PR_2_POINTS simple_exprTession");
		if (($2.atributs.continguts.info_variable.literal == FALSE)
			|| ($4.atributs.continguts.info_variable.literal == FALSE)) {
			
			sinError("Només es permeten expressions literals a la definició de rangs d'enters.");
		}
		
		if (($2.atributs.continguts.info_variable.tipus != TIPUS_INTEGER)
			&& ($4.atributs.continguts.info_variable.tipus != TIPUS_INTEGER)) {
			
			sinError("El limits del rang han de ser de tipus 'integer'.");
		}
		
		if ($2.atributs.continguts.info_variable.valor.integer > $4.atributs.continguts.info_variable.valor.integer) {
			sinError("El limit inferior del rang ha de ser més petit que el limit superior.");
		}
		
		$$.atributs.tipus = TYPE;
		$$.atributs.continguts.info_type.tipus = TIPUS_INTEGER_DERIVAT;
		$$.atributs.continguts.info_type.rang.min = $2.atributs.continguts.info_variable.valor.integer;
		$$.atributs.continguts.info_type.rang.max = $4.atributs.continguts.info_variable.valor.integer;
		$$.atributs.continguts.info_type.rang.tipus = TIPUS_INTEGER;
   }
		
;

simple_expression:
	// NOTE: Només es fan comprovacions de rang pels 'integer'
   '+' term {
	    sinLog(" -> simple_expression -> '+' term ");
		if (($2.atributs.continguts.info_variable.tipus != TIPUS_INTEGER)
			&& ($2.atributs.continguts.info_variable.tipus != TIPUS_INTEGER_DERIVAT)
			&& ($2.atributs.continguts.info_variable.tipus != TIPUS_FLOAT)) {
			sinError("Operador unari '+' no pot ser aplicat a tipus diferent de 'integer' o 'float'.");
		}
		memcpy(&$$, &$2, sizeof(struct Simbol));
   }
   | '-' term {
	    sinLog(" -> simple_expression -> '-' term ");
		if (($2.atributs.continguts.info_variable.tipus != TIPUS_INTEGER)
			&& ($2.atributs.continguts.info_variable.tipus != TIPUS_INTEGER_DERIVAT)
			&& ($2.atributs.continguts.info_variable.tipus != TIPUS_FLOAT)) {
			sinError("Operador unari '-' no pot ser aplicat a tipus diferent de 'integer' o 'float'.");
		}
		memcpy(&$$, &$2, sizeof(struct Simbol));
		
		$$.atributs.continguts.info_variable.literal = FALSE;
		if ($2.atributs.continguts.info_variable.literal) {
			// Invertim signe
			if (($2.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($2.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) {
				// Primer comprovar limit 'integer', si es vol assignar a un integer_derivat
			    // ja es tornara a comprovar amb el seu rang especific al assignment
			    Simbol *symTipus = busca_totes_taules(taulaSimbolActual, "integer");
			    long long int resultat = (long long int)-$2.atributs.continguts.info_variable.valor.integer;
			    if (( resultat > (int)symTipus->atributs.continguts.info_type.rang.max )
					|| ( resultat < (int)symTipus->atributs.continguts.info_type.rang.min )){
				   sinError("El canvi de signe sobrepassa el limit dels 'integers', el resultat ha d'estar entre -2147483647 i 2147483647.");
				}
				$2.atributs.continguts.info_variable.valor.integer = (int)resultat;
			} else if ($2.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) {
				$2.atributs.continguts.info_variable.valor.vfloat = -$2.atributs.continguts.info_variable.valor.vfloat;
			} else {
				sinError("Tipus no vàlid.");
			}
			$$.atributs.continguts.info_variable.literal = TRUE;
		}
   }
   | term {
   		sinLog(" -> simple_expression -> term");
		memcpy(&$$, &$1, sizeof(struct Simbol));
   }
   | simple_expression '+' term {
	   sinLog(" -> simple_expression -> simple_expression '+' term");
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   // Segons els tipus dels elements, el resultat sera integer o float
		   if ( (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT))
			   && (($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) ) {
		       // NOTE: Les operacions amb TIPUS_INTEGER i TIPUS_INTEGER_DERIVAT donen com a resultat sempre un TIPUS_INTEGER
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_INTEGER;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   
			   // Primer comprovar limit 'integer', si es vol assignar a un integer_derivat
			   // ja es tornara a comprovar amb el seu rang especific al assignment
			   Simbol *symTipus = busca_totes_taules(taulaSimbolActual, "integer");
			   long long int resultat = (long long int)$1.atributs.continguts.info_variable.valor.integer + $3.atributs.continguts.info_variable.valor.integer;
			   if (( resultat > (int)symTipus->atributs.continguts.info_type.rang.max )
				   	|| ( resultat < (int)symTipus->atributs.continguts.info_type.rang.min )){
				   sinError("La suma sobrepassa el limit dels 'integers', el resultat ha d'estar entre -2147483647 i 2147483647.");
			   }
			   
			   $$.atributs.continguts.info_variable.valor.integer = (int)resultat;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   // NOTE: No es fa comprovació de limits per floats donat els problemes de precisió d'aquests tipus a C
			   $$.atributs.continguts.info_variable.valor.vfloat = $1.atributs.continguts.info_variable.valor.vfloat + $3.atributs.continguts.info_variable.valor.vfloat;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) && ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = $1.atributs.continguts.info_variable.valor.integer + $3.atributs.continguts.info_variable.valor.vfloat;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = $1.atributs.continguts.info_variable.valor.vfloat + $3.atributs.continguts.info_variable.valor.integer;
		   } else {
			   sinError("L'operador '+' no pot ser aplicat a tipus de dades diferent de 'integer' o 'float'");
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | simple_expression '-' term {
	   sinLog(" -> simple_expression -> simple_expression '-' term");
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   // Segons els tipus dels elements, el resultat sera integer o float
		   if ( (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT))
			   && (($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) ) {
		       // NOTE: Les operacions amb TIPUS_INTEGER i TIPUS_INTEGER_DERIVAT donen com a resultat sempre un TIPUS_INTEGER
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_INTEGER;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   // Primer comprovar limit 'integer', si es vol assignar a un integer_derivat
			   // ja es tornara a comprovar amb el seu rang especific al assignment
			   Simbol *symTipus = busca_totes_taules(taulaSimbolActual, "integer");
			   long long int resultat = (long long int)$1.atributs.continguts.info_variable.valor.integer - $3.atributs.continguts.info_variable.valor.integer;
			   if (( resultat > (int)symTipus->atributs.continguts.info_type.rang.max )
				   	|| ( resultat < (int)symTipus->atributs.continguts.info_type.rang.min )){
				   sinError("La resta sobrepassa el limit dels 'integers', el resultat ha d'estar entre -2147483647 i 2147483647.");
			   }
			   
			   $$.atributs.continguts.info_variable.valor.integer = (int)resultat;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = $1.atributs.continguts.info_variable.valor.vfloat - $3.atributs.continguts.info_variable.valor.vfloat;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) && ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = $1.atributs.continguts.info_variable.valor.integer - $3.atributs.continguts.info_variable.valor.vfloat;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = $1.atributs.continguts.info_variable.valor.vfloat - $3.atributs.continguts.info_variable.valor.integer;
		   } else {
			   sinError("L'operador '-' no pot ser aplicat a tipus de dades diferent de 'integer' o 'float'");
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | simple_expression '&' term {
	   // Concatenació
	   sinLog(" -> simple_expression -> simple_expression '&' term");

	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {	   
		   if (($1.atributs.continguts.info_variable.tipus == TIPUS_STRING) && ($3.atributs.continguts.info_variable.tipus == TIPUS_STRING)) {
			   // STRING & STRING
			   int totalLength = (unsigned char)($1.atributs.continguts.info_variable.valor.string[0]) + (unsigned char)($3.atributs.continguts.info_variable.valor.string[0]);
			   if ((totalLength) > 255) {
				   sinError("La concatenació excedeix el 255 caracters.");
			   }
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_STRING;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   // NOTE: Comprovar que es realitza be la concatenacio
			   sprintf($$.atributs.continguts.info_variable.valor.string, "%c%s%s", totalLength, &($1.atributs.continguts.info_variable.valor.string[1]), &($3.atributs.continguts.info_variable.valor.string[1]));
			   
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_STRING) && ($3.atributs.continguts.info_variable.tipus == TIPUS_CHARACTER)) {
			   // STRING & CHARACTER
			   int totalLength = (unsigned char)($1.atributs.continguts.info_variable.valor.string[0]) + 1;
			   if (totalLength > 255) {
				   sinError("La concatenació excedeix el 255 caracters.");
			   }
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_STRING;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   // NOTE: Comprovar que es realitza be la concatenacio
			   sprintf($$.atributs.continguts.info_variable.valor.string, "%c%s%c", totalLength, &($1.atributs.continguts.info_variable.valor.string[1]), $3.atributs.continguts.info_variable.valor.character);
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
;

term:
   // No es necessari un term_element i desglossar fins els operadors
   factor { 
	    sinLog(" -> term -> factor");
		memcpy(&$$, &$1, sizeof(struct Simbol));
   }
   | factor '*' term {
	   sinLog(" -> term -> factor '*' term");
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   // Segons els tipus dels elements, el resultat sera integer o float
		   if ( (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT))
			   && (($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) ) {
		       // NOTE: Les operacions amb TIPUS_INTEGER i TIPUS_INTEGER_DERIVAT donen com a resultat sempre un TIPUS_INTEGER
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_INTEGER;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   // Primer comprovar limit 'integer', si es vol assignar a un integer_derivat
			   // ja es tornara a comprovar amb el seu rang especific al assignment
			   Simbol *symTipus = busca_totes_taules(taulaSimbolActual, "integer");
			   long long int resultat = (long long int)$1.atributs.continguts.info_variable.valor.integer * $3.atributs.continguts.info_variable.valor.integer;
			   if (( resultat > (int)symTipus->atributs.continguts.info_type.rang.max )
				   	|| ( resultat < (int)symTipus->atributs.continguts.info_type.rang.min )){
				   sinError("La multiplicació sobrepassa el limit dels 'integers', el resultat ha d'estar entre -2147483647 i 2147483647.");
			   }
			   $$.atributs.continguts.info_variable.valor.integer = (int)resultat;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = $1.atributs.continguts.info_variable.valor.vfloat * $3.atributs.continguts.info_variable.valor.vfloat;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) && ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT)) {
			   printf("Avís: Multiplicació de tipus diferents (integer / float).");
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = $1.atributs.continguts.info_variable.valor.integer * $3.atributs.continguts.info_variable.valor.vfloat;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER)) {
			   printf("Avís: Multiplicació de tipus diferents (float / integer).");
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = $1.atributs.continguts.info_variable.valor.vfloat * $3.atributs.continguts.info_variable.valor.integer;
		   } else {
			   sinError("L'operador '*' no pot ser aplicat a tipus de dades diferent de 'integer' o 'float'");
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   
   }
   | factor '/' term {
	   sinLog(" -> term -> factor '/' term");
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   // Controlem la divisió entre 0
		   if (($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER)
		       || ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)){
			   if ($3.atributs.continguts.info_variable.valor.integer == 0) {
				   sinError("Divisió entre 0");
			   }
		   } else if  ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) {
			   if ($3.atributs.continguts.info_variable.valor.vfloat == 0) {
				   sinError("Divisió entre 0");
			   }
		   }
		   
		   double double_tmp;
		   double divisio;
		   
		   // NOTE: No controlem limits de la divisió
		   if ( (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT))
			   	&& (($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) ) {
		   		divisio = $1.atributs.continguts.info_variable.valor.integer / $3.atributs.continguts.info_variable.valor.integer;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT)) {
			   divisio = $1.atributs.continguts.info_variable.valor.vfloat / $3.atributs.continguts.info_variable.valor.vfloat;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) && ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT)) {
			   printf("Avís: Divisió de tipus diferents (integer / float).");
			   divisio = $1.atributs.continguts.info_variable.valor.integer / $3.atributs.continguts.info_variable.valor.vfloat;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER)) {
			   printf("Avís: Divisió de tipus diferents (float / integer).");
			   divisio = $1.atributs.continguts.info_variable.valor.vfloat / $3.atributs.continguts.info_variable.valor.integer;
		   } else {
			   sinError("L'operador '/' no pot ser aplicat a tipus de dades diferent de 'integer' o 'float'");
		   }
		   
		   // Segons el resultat de la divisió, el tipus sera integer o float
		   if (modf(divisio, &double_tmp) > 0){
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = divisio;
		   } else {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_INTEGER;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.integer = divisio;
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | factor PR_MOD term {
	   	sinLog(" -> term -> factor PR_MOD term");
	    if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
			// Controlem la divisió entre 0
			if (($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER)
		       || ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)){
			   if ($3.atributs.continguts.info_variable.valor.integer == 0) {
				   sinError("Divisió entre 0");
			   }
			} else if  ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) {
			   if ($3.atributs.continguts.info_variable.valor.vfloat == 0) {
				   sinError("Divisió entre 0");
			   }
		   	}
		   
			if ( (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT))
			   && (($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) ) {
		       // NOTE: Les operacions amb TIPUS_INTEGER i TIPUS_INTEGER_DERIVAT donen com a resultat sempre un TIPUS_INTEGER
				$$.atributs.tipus = VARIABLE;
				$$.atributs.continguts.info_variable.tipus = TIPUS_INTEGER;
				$$.atributs.continguts.info_variable.literal = TRUE;
				// Primer comprovar limit 'integer', si es vol assignar a un integer_derivat
			    // ja es tornara a comprovar amb el seu rang especific al assignment
			    Simbol *symTipus = busca_totes_taules(taulaSimbolActual, "integer");
			    long long int resultat = (long long int)$1.atributs.continguts.info_variable.valor.integer % $3.atributs.continguts.info_variable.valor.integer;
			    if (( resultat > (int)symTipus->atributs.continguts.info_type.rang.max )
					|| ( resultat < (int)symTipus->atributs.continguts.info_type.rang.min )){
				   sinError("El modul sobrepassa el limit dels 'integers', el resultat ha d'estar entre -2147483647 i 2147483647.");
			    }
				$$.atributs.continguts.info_variable.valor.integer = (int)resultat;
			} else {
				sinError("L'operador MOD només pot ser aplicat a tipus 'integer' mod 'integer'");
			}
		} else {
			$$.atributs.tipus = VARIABLE;
		    $$.atributs.continguts.info_variable.literal = FALSE;
		    sinLog("\t** No son literals.");
		}
   }
;

factor:
   primary { 
	   sinLog(" -> factor -> primary");
	   memcpy(&$$, &$1, sizeof(struct Simbol));
   }
   | primary PR_EXP primary {
	   sinLog(" -> factor -> primary PR_EXP primary");
	   
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   // Segons els tipus dels elements, el resultat sera integer o float
		   if ( (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT))
			   && (($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) ) {
		       // NOTE: Les operacions amb TIPUS_INTEGER i TIPUS_INTEGER_DERIVAT donen com a resultat sempre un TIPUS_INTEGER
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_INTEGER;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   // Primer comprovar limit 'integer', si es vol assignar a un integer_derivat
			   // ja es tornara a comprovar amb el seu rang especific al assignment
			   Simbol *symTipus = busca_totes_taules(taulaSimbolActual, "integer");
			   long long int resultat = (long long int)pow($1.atributs.continguts.info_variable.valor.integer, $3.atributs.continguts.info_variable.valor.integer);
			   if (( resultat > (int)symTipus->atributs.continguts.info_type.rang.max )
					|| ( resultat < (int)symTipus->atributs.continguts.info_type.rang.min )){
				   sinError("La exponenciació sobrepassa el limit dels 'integers', el resultat ha d'estar entre -2147483647 i 2147483647.");
			   }
			   $$.atributs.continguts.info_variable.valor.integer = (int)resultat;
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = pow($1.atributs.continguts.info_variable.valor.vfloat, $3.atributs.continguts.info_variable.valor.vfloat);
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) && ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = pow($1.atributs.continguts.info_variable.valor.integer, $3.atributs.continguts.info_variable.valor.vfloat);
		   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = pow($1.atributs.continguts.info_variable.valor.vfloat, $3.atributs.continguts.info_variable.valor.integer);
		   } else {
			   sinError("L'operador EXP no pot ser aplicat a tipus de dades diferent de 'integer' o 'float'");
		   }
		   
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
	   
		
   }
   | PR_ABS primary {
	   sinLog(" -> factor -> PR_ABS primary");
	   
	   if ($2.atributs.continguts.info_variable.literal) {
		   // Segons els tipus dels elements, el resultat sera integer o float
		   if (($2.atributs.continguts.info_variable.tipus == TIPUS_INTEGER)
			   || ($2.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)){
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_INTEGER;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.integer = abs($2.atributs.continguts.info_variable.valor.integer);
		   } else if ($2.atributs.continguts.info_variable.tipus == TIPUS_FLOAT) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   $$.atributs.continguts.info_variable.valor.vfloat = abs($2.atributs.continguts.info_variable.valor.vfloat);
		   } else {
			   sinError("L'operador ABS no pot ser aplicat a tipus de dades diferent de 'integer' o 'float'");
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | PR_NOT primary {
	   sinLog(" -> factor -> PR_NOT primary");
	   
	   if ($2.atributs.continguts.info_variable.literal) {
		   // Només aplicable a booleans
		   if ($2.atributs.continguts.info_variable.tipus == TIPUS_BOOLEAN) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($2.atributs.continguts.info_variable.valor.integer) {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   }
		   } else {
			   sinError("L'operador ABS no pot ser aplicat a tipus de dades diferent de 'boolean'");
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
;

primary:
   PR_DECIMAL_LITERAL { 
	    sinLog(" -> primary -> DECIMAL LITERAL");
		$$.atributs.tipus = VARIABLE;
		$$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
		$$.atributs.continguts.info_variable.valor.vfloat = $1;
		sprintf($$.nom, "");	// No esta a la TS
		$$.atributs.continguts.info_variable.literal = TRUE;
		$$.atributs.continguts.info_variable.inicialitzat = TRUE;
   }
   | PR_NUMERIC_LITERAL { 
	    sinLog(" -> primary -> NUMERIC LITERAL");
		$$.atributs.tipus = VARIABLE;
		$$.atributs.continguts.info_variable.tipus = TIPUS_INTEGER;
		$$.atributs.continguts.info_variable.valor.integer = (int)$1;
		sprintf($$.nom, "");	// No esta a la TS
		$$.atributs.continguts.info_variable.literal = TRUE;
		$$.atributs.continguts.info_variable.inicialitzat = TRUE;
   }
   | PR_STRING_LITERAL { 
	   sinLog(" -> primary -> STRING LITERAL");
	   $$.atributs.tipus = VARIABLE;
	   $$.atributs.continguts.info_variable.tipus = TIPUS_STRING;
	   
	   if (strlen($1)-2 > 255) {  // - 1 (longitud) -1 (final string \0)
			sinError("String literal \"%s\" massa llarg. Màxim 255 caracters i té %d caracters", $1, strlen($1));
	   }
	   memcpy(&($$.atributs.continguts.info_variable.valor.string), $1, strlen($1));
	   $$.atributs.continguts.info_variable.literal = TRUE;
	   $$.atributs.continguts.info_variable.inicialitzat = TRUE;
	   sprintf($$.nom, "");	// No esta a la TS
   }
   | name {
	   sinLog(" -> primary -> name");
	   memcpy(&$$, &$1,sizeof(struct Simbol));
   }
   | '(' expression ')' {
	   sinLog(" -> primary -> expression");
	   memcpy(&$$, &$2,sizeof(struct Simbol));
   }
;

name:
   direct_name {
	   // Retorna VARIABLE
	    sinLog(" -> name -> direct_name");
		memcpy(&$$, &$1,sizeof(struct Simbol));
   }
   | indexed_component {
	   // Retorna VARIABLE
	   sinLog(" -> name -> indexed_component");
	   memcpy(&$$, &$1,sizeof(struct Simbol));
   }
   | selected_component {
	   // Retorna VARIABLE
	   sinLog(" -> name -> selected_component");
	   memcpy(&$$, &$1,sizeof(struct Simbol));
   }
   | type_conversion {
	   // Retorna VARIABLE
	   sinLog(" -> name -> type_conversion");
	   memcpy(&$$, &$1,sizeof(struct Simbol));
   }
   | function_call {
	   // Retorna VARIABLE
	   sinLog(" -> name -> function_call");
	   memcpy(&$$, &$1,sizeof(struct Simbol));
   }
   | PR_CHARACTER_LITERAL {
	    sinLog(" -> name -> PR_CHARACTER_LITERAL");
		$$.atributs.tipus = VARIABLE;
		$$.atributs.continguts.info_variable.tipus = TIPUS_CHARACTER;
		$$.atributs.continguts.info_variable.valor.character = $1;
		$$.atributs.continguts.info_variable.literal = TRUE;
		sprintf($$.nom, "");	// No esta a la TS
   }
;

function_call:
	// Aquí es produeix un "Expected shift/reduce conflict"
	PR_ID_FUNCTION {
		Boolean trobat = FALSE;
		Info_Function *info_function;
		
		info_function = &($1->atributs.continguts.info_function);
		if (info_function->num_parametres == 0) {
			trobat = TRUE;
		}
		
		if (trobat == FALSE) {
			if ($1->atributs.continguts.info_function.sobrecarregues == NULL) {
				sinError("La funció '%s' requereix '%i' paràmetre(s).", $1->nom, $1->atributs.continguts.info_function.num_parametres);
			} else {
				// Recorrer sobrecarregues
				reiniciar_PDI($1->atributs.continguts.info_function.sobrecarregues);
				info_function = (Info_Function *)obtenir_element_PDI($1->atributs.continguts.info_function.sobrecarregues);
				while(info_function != NULL) {
					// Comprovar parametres
					if (info_function->num_parametres == 0) {
						trobat = TRUE;
						break;
					}
					
					seguent_PDI($1->atributs.continguts.info_function.sobrecarregues);
					info_function = (Info_Function *)obtenir_element_PDI($1->atributs.continguts.info_function.sobrecarregues);
				}
			}
		}
		
		if (trobat) {
			// NOTE: Es pot cridar una funcio amb cos no definit si la signatura si ha sigut definida
			// Al final del compilador ja es comprova que tots tenen cos definit.
			/*if (info_function->complet == FALSE) {
				sinError("La funció '%s' no te encara el cos definit.", $1->nom);
			}*/
			
			$$.atributs.tipus = VARIABLE;
			$$.atributs.continguts.info_variable.tipus = info_function->tipus_retorn;
			$$.atributs.continguts.info_variable.literal = FALSE;
			$$.atributs.continguts.info_variable.inicialitzat = FALSE;
		
			if (($1->atributs.continguts.info_function.tipus_retorn == TIPUS_DERIVAT)
				|| ($1->atributs.continguts.info_function.tipus_retorn == TIPUS_INTEGER_DERIVAT)) {
				memcpy($$.atributs.continguts.info_variable.nom_tipus, info_function->nom_tipus_retorn, STRING_LENGTH);
			}
		} else {
			sinError("La funció '%s' no te cap sobrecarrega que permeti 0 paràmetres.", $1->nom);
		}
	}
	| PR_ID_FUNCTION function_call_element {
		
		Boolean trobat = FALSE;
		Info_Function *info_function;
		
		info_function = &($1->atributs.continguts.info_function);
		// Mateix nombre de parametres?

		if (info_function->num_parametres == contador) {
			// Comprovar tipus parametres
			if (crida_parametres_diferents(cua_tmp, info_function->parametres, info_function->taula) == FALSE) {
				// Coincideixen el tipus dels parametres amb la definicio
				trobat = TRUE;
			}
		}
		
		if (trobat == FALSE) {
			if ($1->atributs.continguts.info_function.sobrecarregues == NULL) {
				sinError("La funció '%s' requereix '%i' paràmetre(s) i no s'han trobat cap tipus coincident.", $1->nom, $1->atributs.continguts.info_function.num_parametres);
			} else {
				// Recorrer sobrecarregues
				reiniciar_PDI($1->atributs.continguts.info_function.sobrecarregues);
				info_function = (Info_Function *)obtenir_element_PDI($1->atributs.continguts.info_function.sobrecarregues);
				while(info_function != NULL) {
					// Mateix nombre de parametres?
					if (info_function->num_parametres == contador) {
						// Comprovar tipus parametres
						if (crida_parametres_diferents(cua_tmp, info_function->parametres, info_function->taula) == FALSE) {
							// Coincideixen el tipus dels parametres amb la definicio
							trobat = TRUE;
						}
						break;
					}
					
					seguent_PDI($1->atributs.continguts.info_function.sobrecarregues);
					info_function = (Info_Function *)obtenir_element_PDI($1->atributs.continguts.info_function.sobrecarregues);
				}
			}
		}
		
		if (trobat) {
			// NOTE: Es pot cridar una funcio amb cos no definit si la signatura si ha sigut definida
			// Al final del compilador ja es comprova que tots tenen cos definit.
			/*if (info_function->complet == FALSE) {
				sinError("La funció '%s' no te encara el cos definit.", $1->nom);
			}*/
			
			$$.atributs.tipus = VARIABLE;
			$$.atributs.continguts.info_variable.tipus = info_function->tipus_retorn;
			$$.atributs.continguts.info_variable.literal = FALSE;
			$$.atributs.continguts.info_variable.inicialitzat = FALSE;
		
			if (($1->atributs.continguts.info_function.tipus_retorn == TIPUS_DERIVAT)
				|| ($1->atributs.continguts.info_function.tipus_retorn == TIPUS_INTEGER_DERIVAT)) {
				memcpy($$.atributs.continguts.info_variable.nom_tipus, info_function->nom_tipus_retorn, STRING_LENGTH);
			}
			
			cua_tmp = NULL;
		} else {
			sinError("La funció '%s' no te cap sobrecarrega que permeti '%i' paràmetre(s) amb aquests tipus.", $1->nom, contador);
		}
	}
;

function_call_element:
	'(' {
		cua_tmp = (Cua *)crear_cua();
		contador = 0;
	}
	actual_parameter_part_element ')' {}
;

actual_parameter_part_element: // Gramatica auxiliar de 'actual_parameter_part'
	parameter_association {}
   | actual_parameter_part_element ',' parameter_association  {}
;


parameter_association:
   explicit_actual_parameter {}
;

explicit_actual_parameter:
   expression {
	   sinLog(" -> explicit_actual_parameter -> expression");
	   
	   // Cua d'arguments de la crida d'una funcio o procediment
	   Simbol *sym = (Simbol *)malloc(sizeof(struct Simbol));
	   memcpy(sym, &$1, sizeof(struct Simbol));
	   encuar(cua_tmp, sym);
	   contador++; // Nombre de parámetres a la crida a la funcio/procediment
   }
   //| PR_ID_VARIABLE {}  //esta incluida en la anterior ?? Segun la practica vieja si.
;
	

type_conversion:
	subtype_mark '(' name ')' {
		sinLog(" -> type_conversion -> subtype_mark '(' name ')'");
		if (($1.atributs.tipus == TYPE) && ($3.atributs.tipus == VARIABLE)) {
			$$.atributs.tipus = VARIABLE;
			$$.atributs.continguts.info_variable.literal = $3.atributs.continguts.info_variable.literal;
			if (($1.atributs.continguts.info_type.tipus == TIPUS_CHARACTER) 
				&& (($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT))) {
				// INTEGER||INTEGER_DERIVAT -> CHARACTER				
			    $$.atributs.continguts.info_variable.tipus = TIPUS_CHARACTER;
				$$.atributs.continguts.info_variable.valor.character = (char)$3.atributs.continguts.info_variable.valor.integer;
			} else if ((($1.atributs.continguts.info_type.tipus == TIPUS_INTEGER) || ($1.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT))
				&& ($3.atributs.continguts.info_variable.tipus == TIPUS_CHARACTER)) {
				// CHARACTER -> INTEGER||INTEGER_DERIVAT
			    $$.atributs.continguts.info_variable.tipus = TIPUS_INTEGER;
				$$.atributs.continguts.info_variable.valor.integer = $3.atributs.continguts.info_variable.valor.character;
			} else if ((($1.atributs.continguts.info_type.tipus == TIPUS_INTEGER) || ($1.atributs.continguts.info_type.tipus == TIPUS_INTEGER_DERIVAT)) 
				&& ($3.atributs.continguts.info_variable.tipus == TIPUS_FLOAT)) {
				// FLOAT -> INTEGER||INTEGER_DERIVAT
			    $$.atributs.continguts.info_variable.tipus = TIPUS_INTEGER;
				$$.atributs.continguts.info_variable.valor.integer = (int)$3.atributs.continguts.info_variable.valor.vfloat;
			} else if (($1.atributs.continguts.info_type.tipus == TIPUS_FLOAT) 
				&& (($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER))) {
				// INTEGER||INTEGER_DERIVAT -> FLOAT
			    $$.atributs.continguts.info_variable.tipus = TIPUS_FLOAT;
				$$.atributs.continguts.info_variable.valor.vfloat = $3.atributs.continguts.info_variable.valor.integer;
			} else if (($1.atributs.continguts.info_type.tipus == TIPUS_STRING) && ($3.atributs.continguts.info_variable.tipus == TIPUS_CHARACTER)) {
				// CHARACTER -> STRING
			    $$.atributs.continguts.info_variable.tipus = TIPUS_STRING;
				sprintf($$.atributs.continguts.info_variable.valor.string, "%c", $3.atributs.continguts.info_variable.valor.character);
			} else {
				sinError("La conversió no es pot realitzar");
			}
		} else {
			sinError("La conversió no es pot realitzar");
		}
	}
;


selected_component:
   prefix '.' selector_name {
	   sinLog(" -> selected_component -> name '.' PR_ID");
	   if ($1.atributs.tipus != VARIABLE) {
		   sinError("Prefix ha de ser una variable");
	   }
	   if ($1.atributs.continguts.info_variable.tipus != TIPUS_DERIVAT) {
		   sinError("Prefix ha de ser tipus RECORD.");
	   }
	   
	   // Busquem el simbol del tipus del "prefix"
	   Simbol *symTipus = busca_totes_taules(taulaSimbolActual, $1.atributs.continguts.info_variable.nom_tipus);
	   if ((symTipus == NULL) || (symTipus->atributs.tipus != RECORD_TYPE)) {
		   sinError("No es possible tractar amb un RECORD '%s' donat que no es d'aquest tipus.", $1.atributs.continguts.info_variable.nom_tipus);
	   }
	   
	   // Busquem entre els elements del RECORD per veure si existeix selector_name
	   Simbol *symSubtipus = obtenir_simbol(symTipus->atributs.continguts.info_record.elements, $3);
	   if (symSubtipus == NULL) {
		   sinError("%s no es un membre del RECORD tipus '%s'", $3, $1.atributs.continguts.info_variable.nom_tipus);
	   }
	   // Retornem el simbol del tipus dintre del record
	   memcpy(&$$, symSubtipus, sizeof(struct Simbol));
   }
;

selector_name:
   PR_ID {
	   sinLog(" -> selector_name -> PR_ID");
	   snprintf($$, STRING_LENGTH, "%s" , $1);
   }
;

direct_name:
   PR_ID_VARIABLE {
	    sinLog(" -> direct_name -> PR_ID_VARIABLE");
		memcpy(&$$, $1,sizeof(struct Simbol));
		$$.atributs.continguts.info_variable.literal = FALSE;
   }
;

prefix:
   name {
	    sinLog(" -> prefix -> name");
		memcpy(&$$, &$1,sizeof(struct Simbol));
   }
;

indexed_component:
   prefix '(' expression ')' {
	   
	   sinLog(" -> indexed_component -> prefix '(' expression ')'");
	   
	   // Es suposa que en prefix hi ha una VARIABLE
	   if ($1.atributs.tipus != VARIABLE) {
		   sinError("L'index ha de ser una variable.");
	   }
	   
	   // Es suposa que en prefix hi ha una VARIABLE del tipus ARRAY
	   if ($1.atributs.continguts.info_variable.tipus != TIPUS_DERIVAT) {
		   sinError("No es possible tractar amb un ARRAY '%s' donat que no es d'aquest tipus.", $1.atributs.continguts.info_variable.nom_tipus);
	   }
	   
	   // Busquem el simbol del tipus de la variable
	   Simbol *symTipus = busca_totes_taules(taulaSimbolActual, $1.atributs.continguts.info_variable.nom_tipus);
	   if ((symTipus == NULL) || (symTipus->atributs.tipus != ARRAY_TYPE)) {
		   sinError("No es possible tractar amb un ARRAY '%s' donat que no es d'aquest tipus.", $1.atributs.continguts.info_variable.nom_tipus);
	   }
	   
	   if ($3.atributs.tipus != VARIABLE) {
		   sinError("L'index ha de ser una variable.");
	   }
	   
	   // Comprovem que hi esta dintre del rang
	   if ($3.atributs.continguts.info_variable.literal) {
		   if (($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER) || ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) {
			   if (($3.atributs.continguts.info_variable.valor.integer < (int)symTipus->atributs.continguts.info_array.rang.min)
				   || ($3.atributs.continguts.info_variable.valor.integer > (int)symTipus->atributs.continguts.info_array.rang.max)) {
				   sinError("Index fora de rang.");
			   }
		   } else if ($3.atributs.continguts.info_variable.tipus == TIPUS_CHARACTER) {
			   if (($3.atributs.continguts.info_variable.valor.character < (int)symTipus->atributs.continguts.info_array.rang.min)
				   || ($3.atributs.continguts.info_variable.valor.character > (int)symTipus->atributs.continguts.info_array.rang.max)) {
				   sinError("Index fora de rang.");
			   }
		   } else {
			   sinError("El tipus de l'index d'un array només pot ser 'integer' o 'character'.");
		   }
	   } else {
		   /*$$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.literal = FALSE;*/
		   sinLog("\t** No son literals.");
	   }
	   
	   /*// Retornem el tipus (ARRAY) de la variable
	   memcpy(&$$, symTipus, sizeof(struct Simbol));*/
	   
	   // Retornem la VARIABLE, faltaria retornar l'index... pero no cal per ara.
	   memcpy(&$$, &$1, sizeof(struct Simbol));
   }
;


expression:   
   relation {
	   sinLog(" -> expression -> relation");
	   memcpy(&$$, &$1, sizeof(struct Simbol));
   }
   | expression PR_AND relation {
	   if (($1.atributs.continguts.info_variable.tipus != TIPUS_BOOLEAN) 
		   || ($3.atributs.continguts.info_variable.tipus != TIPUS_BOOLEAN)) {
	   		sinError("L'operador AND ha de trobar-se entre tipus booleans");
	   }
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = TRUE;
		   if (($1.atributs.continguts.info_variable.valor.boolean == TRUE) && ($3.atributs.continguts.info_variable.valor.boolean == TRUE)) {
			   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
		   } else {
			   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | expression PR_OR relation {
	   if (($1.atributs.continguts.info_variable.tipus != TIPUS_BOOLEAN) 
		   || ($3.atributs.continguts.info_variable.tipus != TIPUS_BOOLEAN)) {
	   		sinError("L'operador AND ha de trobar-se entre tipus booleans");
	   }
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = TRUE;
		   if (($1.atributs.continguts.info_variable.valor.boolean == TRUE) || ($3.atributs.continguts.info_variable.valor.boolean == TRUE)) {
			   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
		   } else {
			   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | expression PR_AND PR_THEN relation {
	   // e_left and then e_right: s'avalua e_left i, si és certa, s'avalua e_right
	   if (($1.atributs.continguts.info_variable.tipus != TIPUS_BOOLEAN) 
		   || ($4.atributs.continguts.info_variable.tipus != TIPUS_BOOLEAN)) {
	   		sinError("L'operador AND ha de trobar-se entre tipus booleans");
	   }
	   if (($1.atributs.continguts.info_variable.literal) && ($4.atributs.continguts.info_variable.literal)) {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = TRUE;
		   if ($1.atributs.continguts.info_variable.valor.boolean == TRUE) {
			   if ($4.atributs.continguts.info_variable.valor.boolean == TRUE) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else {
			   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | expression PR_OR PR_ELSE relation {
	   // e_left or else e_right: s'avalua e_left i, si és falsa, s'avalua e_right
	   if (($1.atributs.continguts.info_variable.tipus != TIPUS_BOOLEAN) 
		   || ($4.atributs.continguts.info_variable.tipus != TIPUS_BOOLEAN)) {
	   		sinError("L'operador AND ha de trobar-se entre tipus booleans");
	   }
	   if (($1.atributs.continguts.info_variable.literal) && ($4.atributs.continguts.info_variable.literal)) {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = TRUE;
		   if ($1.atributs.continguts.info_variable.valor.boolean == FALSE) {
			   if ($4.atributs.continguts.info_variable.valor.boolean == TRUE) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else {
			   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
;

relation:
   simple_expression {
	    sinLog(" -> relation -> simple_expression");
		memcpy(&$$, &$1,sizeof(struct Simbol));
   }
   | simple_expression '=' simple_expression {
	   sinLog(" -> relation -> simple_expression '=' simple_expression");
	   if ($1.atributs.tipus != $3.atributs.tipus) {
		   sinError("Les expressions han de ser del mateix tipus.");
	   }
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   if (($1.atributs.tipus == TIPUS_INTEGER) || ($1.atributs.tipus == TIPUS_INTEGER_DERIVAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.integer == $3.atributs.continguts.info_variable.valor.integer) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_FLOAT) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.vfloat == $3.atributs.continguts.info_variable.valor.vfloat) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_CHARACTER) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.character == $3.atributs.continguts.info_variable.valor.character) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_STRING) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if (strcmp($1.atributs.continguts.info_variable.valor.string, $3.atributs.continguts.info_variable.valor.string) == 0) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | simple_expression '<' simple_expression {
	   sinLog(" -> relation -> simple_expression '<' simple_expression");
	   if ($1.atributs.tipus != $3.atributs.tipus) {
		   sinError("Les expressions han de ser del mateix tipus.");
	   }
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   if (($1.atributs.tipus == TIPUS_INTEGER) || ($1.atributs.tipus == TIPUS_INTEGER_DERIVAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.integer < $3.atributs.continguts.info_variable.valor.integer) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_FLOAT) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.vfloat < $3.atributs.continguts.info_variable.valor.vfloat) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_CHARACTER) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.character < $3.atributs.continguts.info_variable.valor.character) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_STRING) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if (strcmp($1.atributs.continguts.info_variable.valor.string, $3.atributs.continguts.info_variable.valor.string) < 0) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | simple_expression '>' simple_expression {
	   sinLog(" -> relation -> simple_expression '>' simple_expression");
	   if ($1.atributs.tipus != $3.atributs.tipus) {
		   sinError("Les expressions han de ser del mateix tipus.");
	   }
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   if (($1.atributs.tipus == TIPUS_INTEGER) || ($1.atributs.tipus == TIPUS_INTEGER_DERIVAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.integer > $3.atributs.continguts.info_variable.valor.integer) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_FLOAT) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.vfloat > $3.atributs.continguts.info_variable.valor.vfloat) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_CHARACTER) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.character > $3.atributs.continguts.info_variable.valor.character) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_STRING) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if (strcmp($1.atributs.continguts.info_variable.valor.string, $3.atributs.continguts.info_variable.valor.string) > 0) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | simple_expression PR_DIFFERENT simple_expression {
	   sinLog(" -> relation -> simple_expression '!=' simple_expression");
	   if ($1.atributs.tipus != $3.atributs.tipus) {
		   sinError("Les expressions han de ser del mateix tipus.");
	   }
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   if (($1.atributs.tipus == TIPUS_INTEGER) || ($1.atributs.tipus == TIPUS_INTEGER_DERIVAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.integer != $3.atributs.continguts.info_variable.valor.integer) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_FLOAT) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.vfloat != $3.atributs.continguts.info_variable.valor.vfloat) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_CHARACTER) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.character != $3.atributs.continguts.info_variable.valor.character) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_STRING) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if (strcmp($1.atributs.continguts.info_variable.valor.string, $3.atributs.continguts.info_variable.valor.string) != 0) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | simple_expression PR_MINEQU simple_expression {
	   sinLog(" -> relation -> simple_expression '<=' simple_expression");
	   if ($1.atributs.tipus != $3.atributs.tipus) {
		   sinError("Les expressions han de ser del mateix tipus.");
	   }
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   if (($1.atributs.tipus == TIPUS_INTEGER) || ($1.atributs.tipus == TIPUS_INTEGER_DERIVAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.integer <= $3.atributs.continguts.info_variable.valor.integer) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_FLOAT) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.vfloat <= $3.atributs.continguts.info_variable.valor.vfloat) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_CHARACTER) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.character <= $3.atributs.continguts.info_variable.valor.character) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_STRING) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if (strcmp($1.atributs.continguts.info_variable.valor.string, $3.atributs.continguts.info_variable.valor.string) <= 0) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | simple_expression PR_MAJEQU simple_expression {
	   sinLog(" -> relation -> simple_expression '>=' simple_expression");
	   if ($1.atributs.tipus != $3.atributs.tipus) {
		   sinError("Les expressions han de ser del mateix tipus.");
	   }
	   if (($1.atributs.continguts.info_variable.literal) && ($3.atributs.continguts.info_variable.literal)) {
		   if (($1.atributs.tipus == TIPUS_INTEGER) || ($1.atributs.tipus == TIPUS_INTEGER_DERIVAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.integer >= $3.atributs.continguts.info_variable.valor.integer) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_FLOAT) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.vfloat >= $3.atributs.continguts.info_variable.valor.vfloat) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_CHARACTER) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if ($1.atributs.continguts.info_variable.valor.character >= $3.atributs.continguts.info_variable.valor.character) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_STRING) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if (strcmp($1.atributs.continguts.info_variable.valor.string, $3.atributs.continguts.info_variable.valor.string) >= 0) {
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   
   | simple_expression PR_IN range {
	   sinLog(" -> relation -> simple_expression PR_IN range");
	   if ($1.atributs.tipus != $3.atributs.continguts.info_array.rang.tipus) {
		   sinError("El rang no es del mateix tipus que l'element comparat.");
	   }
	   if ($1.atributs.continguts.info_variable.literal) {
		   if (($1.atributs.tipus == TIPUS_INTEGER) || ($1.atributs.tipus == TIPUS_INTEGER_DERIVAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if (($1.atributs.continguts.info_variable.valor.integer >= (int)$3.atributs.continguts.info_array.rang.min) 
			   		&& ($1.atributs.continguts.info_variable.valor.integer <= (int)$3.atributs.continguts.info_array.rang.max)){
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_FLOAT) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if (($1.atributs.continguts.info_variable.valor.vfloat >= $3.atributs.continguts.info_array.rang.min) 
			   		&& ($1.atributs.continguts.info_variable.valor.vfloat <= $3.atributs.continguts.info_array.rang.max)){
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_CHARACTER) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if (($1.atributs.continguts.info_variable.valor.character >= (int)$3.atributs.continguts.info_array.rang.min) 
			   		&& ($1.atributs.continguts.info_variable.valor.character <= (int)$3.atributs.continguts.info_array.rang.max)){
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else {
			   sinError("Només es poden avaluar en rangs els tipus 'integer', 'float' i 'character'.");
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
   | simple_expression PR_NOT PR_IN range {
	   sinLog(" -> relation -> simple_expression PR_NOT PR_IN range");
	   if ($1.atributs.tipus != $4.atributs.continguts.info_array.rang.tipus) {
		   sinError("El rang no es del mateix tipus que l'element comparat.");
	   }
	   if ($1.atributs.continguts.info_variable.literal) {
		   if (($1.atributs.tipus == TIPUS_INTEGER) || ($1.atributs.tipus == TIPUS_INTEGER_DERIVAT)) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if (($1.atributs.continguts.info_variable.valor.integer < (int)$4.atributs.continguts.info_array.rang.min) 
			   		|| ($1.atributs.continguts.info_variable.valor.integer > (int)$4.atributs.continguts.info_array.rang.max)){
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_FLOAT) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if (($1.atributs.continguts.info_variable.valor.vfloat < $4.atributs.continguts.info_array.rang.min) 
			   		|| ($1.atributs.continguts.info_variable.valor.vfloat > $4.atributs.continguts.info_array.rang.max)){
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else if ($1.atributs.tipus == TIPUS_CHARACTER) {
			   $$.atributs.tipus = VARIABLE;
			   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
			   $$.atributs.continguts.info_variable.literal = TRUE;
			   if (($1.atributs.continguts.info_variable.valor.character < (int)$4.atributs.continguts.info_array.rang.min) 
			   		|| ($1.atributs.continguts.info_variable.valor.character > (int)$4.atributs.continguts.info_array.rang.max)){
				   $$.atributs.continguts.info_variable.valor.boolean = TRUE;
			   } else {
				   $$.atributs.continguts.info_variable.valor.boolean = FALSE;
			   }
		   } else {
			   sinError("Només es poden avaluar en rangs els tipus 'integer', 'float' i 'character'.");
		   }
	   } else {
		   $$.atributs.tipus = VARIABLE;
		   $$.atributs.continguts.info_variable.tipus = TIPUS_BOOLEAN;
		   $$.atributs.continguts.info_variable.literal = FALSE;
		   sinLog("\t** No son literals.");
	   }
   }
;

range:
   simple_expression PR_2_POINTS simple_expression {
	    
	   if ($1.atributs.continguts.info_variable.literal == FALSE || $3.atributs.continguts.info_variable.literal == FALSE) {
			sinError("expressions in range must be literals");
	   }
		sinLog(" -> range -> simple_expression T_2_POINTS simple_expression");
		if ($1.atributs.continguts.info_variable.tipus != $3.atributs.continguts.info_variable.tipus){
			sinError("Limit range types have different types");
		}
		/*if (strcmp($1.name_type,$3.name_type)!=0){
			sinError("Limit range types have different types %s != %s",$1.name_type,$3.name_type);
		}*/
		// Només si es un float, l'ordre ha de ser normal
		if ($1.atributs.continguts.info_variable.valor.vfloat > $3.atributs.continguts.info_variable.valor.vfloat){
			sinError("Second expression must be major or equal to the first expression");
		}
		
		Rang *rang = malloc(sizeof(Rang));
		rang->tipus = $1.atributs.continguts.info_variable.tipus;
		switch (rang->tipus){
			case (TIPUS_FLOAT):
				rang->max = $3.atributs.continguts.info_variable.valor.vfloat;
				rang->min = $1.atributs.continguts.info_variable.valor.vfloat;
				break;
			case (TIPUS_INTEGER):
			case (TIPUS_INTEGER_DERIVAT):
				rang->max = $3.atributs.continguts.info_variable.valor.integer;
				rang->min = $1.atributs.continguts.info_variable.valor.integer;
				break;
			case(TIPUS_CHARACTER):
				rang->max = (int)$3.atributs.continguts.info_variable.valor.character;
				rang->min = (int)$1.atributs.continguts.info_variable.valor.character;
				break;
			default:
				sinError("The Type can not be a range");
		}
		// Passem el rang cap adalt
		sprintf($$.nom, "");	// No és encara un simbol de la TS
		$$.atributs.tipus = ARRAY_TYPE;
		$$.atributs.continguts.info_array.rang.min = rang->min;
		$$.atributs.continguts.info_array.rang.max = rang->max;
		$$.atributs.continguts.info_array.rang.tipus = rang->tipus;
   }
;


enumeration_type_definition:
	// Potser serà necessari introduir codi intermedi despres del '(' per inicialitzar
	// una taula de simbols del enumerador
	'(' {
			cua_tmp = (Cua *)crear_cua();
			contador = 0;
	}
	enumeration_literal_specification enumeration_type_definition_element {
		sinLog(" -> enumeration_type_definition -> '(' aux_enumeration_type_definition ')'");
		$$.atributs.tipus = ENUMERATED_TYPE;
		$$.atributs.continguts.info_enumerated.elements = cua_tmp;
	}
;

enumeration_type_definition_element: /* Gramatica auxiliar de 'enumeration_type_definition' */
   ',' enumeration_literal_specification enumeration_type_definition_element { 
	   sinLog(" -> enumeration_type_definition_element -> seguent element");
   }
   | ')' { sinLog(" -> enumeration_type_definition_element -> tanca parentesis )"); }
;

enumeration_literal_specification:
	// Tractament de casos de identificadors ja definits?
   defining_identifier {
	   	sinLog(" -> enumeration_literal_specification -> defining_identifier");
		sinLog("Nou tipus enumerat %s insertat", $1);
		Simbol *element = (Simbol*)malloc(sizeof(struct Simbol));
		element->atributs.tipus = VARIABLE;
		element->atributs.continguts.info_variable.valor.taula = NULL;
		element->atributs.continguts.info_variable.inicialitzat = TRUE;
		element->atributs.continguts.info_variable.tipus = TIPUS_INTEGER;
		element->atributs.continguts.info_variable.constant = TRUE;
		sprintf(element->nom, $1);
		element->atributs.continguts.info_variable.valor.integer = contador;
		contador++;
		// cua_tmp ha sigut anteriorment inicialitzada en nodes pares
		encuar(cua_tmp, element);
   } 
   | defining_character_literal {
	    // IDEM defining_identifier
	   	sinLog(" -> enumeration_literal_specification -> defining_identifier");
		sinLog("Nou tipus enumerat %s insertat", $1);
		Simbol *element = (Simbol*)malloc(sizeof(struct Simbol));
		element->atributs.tipus = VARIABLE;
		element->atributs.continguts.info_variable.valor.taula = NULL;
		element->atributs.continguts.info_variable.inicialitzat = TRUE;
		element->atributs.continguts.info_variable.tipus = TIPUS_INTEGER;
		element->atributs.continguts.info_variable.constant = TRUE;
		sprintf(element->nom, $1);
		element->atributs.continguts.info_variable.valor.integer = contador;
		contador++;
		// cua_tmp ha sigut anteriorment inicialitzada en nodes pares
		encuar(cua_tmp, element);
   }
   | defined_identifier {} // NOTE: Control de duplicats
;

defining_character_literal:
   PR_CHARACTER_LITERAL {
	   sinLog(" -> defining_character_literal -> PR_CHARACTER_LITERAL");
	   sprintf($$, "%c", $1);
   }
;

defining_identifier:
   PR_ID {
   	sinLog(" -> defining_identifier -> PR_ID");
	// Pasem el nom de l'identificador cap adalt
	snprintf($$, STRING_LENGTH, "%s" , $1);
   }
;

// NOTE: Creat per controlar errors de identificadors ya declarats
defined_identifier:
	PR_ID_VARIABLE {
		sinError("%s ja esta definit anteriorment com una variable", $1->nom);
	}
	| PR_ID_PROCEDURE {
		sinError("%s ja esta definit anteriorment com un procediment", $1->nom);
	}
	| PR_ID_FUNCTION {
		sinError("%s ja esta definit anteriorment com una funció", $1->nom);
	}
	| PR_ID_TYPE {
		sinError("%s ja esta definit anteriorment com un tipus", $1->nom);
	}
;

defining_program_unit_name:
   defining_identifier {
	    sinLog(" -> defining program unit name -> defining identifier");
		// Pasem el nom de l'identificador cap adalt
		snprintf($$, STRING_LENGTH, "%s" , $1);
   }
;

designator:
   PR_ID_PROCEDURE {
	   sinLog(" -> designator -> procedure designator");
	   memcpy(&$$, $1, sizeof(struct Simbol));
   }
   | PR_ID {
	   sinError("Designator '%s' no es correspon amb el nom de la funció/procediment encaixats.", $1);
   }
;

declarative_part:
   {}
   //|declarative_item {} // Redundant
   | declarative_item declarative_part {}
;

declarative_item:
   basic_declarative_item { sinLog(" -> declarative_item -> basic_declarative_item");}
   | body {sinLog(" -> declarative_item -> body");}
;

body:
   proper_body { sinLog(" -> body -> proper_body");}
;

proper_body:
   subprogram_body {sinLog(" -> proper_body -> subprogram_body");}
;

subprogram_body:
	subprogram_body_element declarative_part PR_BEGIN sequence_of_statements PR_END ';' {
	   sinLog("-> subprogram_body -> subprogram_body_element PR_IS declarative_part PR_BEGIN sequence_of_statements PR_END ';'");
	   taulaSimbolActual = taulaSimbolActual->anterior;
	   
	   // A $1 tenim una copia del simbol, hem de buscar l'original per canviar l'atribut 'complet'
	   Simbol *sym = obtenir_simbol(taulaSimbolActual, $1.nom);
	   
	   // Indicar que la funcio/procediment ja esta complet
	   if (sym != NULL) {
		   if (sym->atributs.tipus == PROCEDURE) {
			   int id = $1.atributs.continguts.info_procedure.id; // Identificador de sobrecarrega
			   
			   if (id > 0) {
				   // Busquem la sobrecarrega que correspon
				   reiniciar_PDI(sym->atributs.continguts.info_procedure.sobrecarregues);
				   int i;
				   for (i = 1; i < id; i++) {
						seguent_PDI(sym->atributs.continguts.info_procedure.sobrecarregues);
				   }
				   Info_Procedure *info_procedure = (Info_Procedure *)obtenir_element_PDI(sym->atributs.continguts.info_procedure.sobrecarregues);
				   info_procedure->complet = TRUE;
			   } else {
				   // Principal
				   sym->atributs.continguts.info_procedure.complet = TRUE;
			   }
			   
		   } else if (sym->atributs.tipus == FUNCTION) {
			   int id = $1.atributs.continguts.info_function.id; // Identificador de sobrecarrega
			   
			   if (id > 0) {
				   // Busquem la sobrecarrega que correspon
				   reiniciar_PDI(sym->atributs.continguts.info_function.sobrecarregues);
				   int i;
				   for (i = 1; i < id; i++) {
						seguent_PDI(sym->atributs.continguts.info_function.sobrecarregues);
				   }
				   
				   Info_Function *info_function = (Info_Function *)obtenir_element_PDI(sym->atributs.continguts.info_function.sobrecarregues);

				   info_function->complet = TRUE;
				   
			   } else {
				   // Principal
				   sym->atributs.continguts.info_function.complet = TRUE;
			   }
		   } else {
			   sinError("S'esperava PROCEDURE o FUNCTION.");
		   }
	   } else {
		   sinError("Simbol no trobat.");
	   }
   }
   | subprogram_body_element declarative_part PR_BEGIN sequence_of_statements PR_END designator ';' { 
	   sinLog("-> subprogram_body -> subprogram_body_element PR_IS declarative_part PR_BEGIN sequence_of_statements PR_END ';'");

	   // Comprovem designator
	   if (strcmp($1.nom, $6.nom) != 0) {
		   sinError("Designator '%s' no es correspon amb el nom de la funció/procediment %s.", $6.nom, $1.nom);
	   }
	   
	   taulaSimbolActual = taulaSimbolActual->anterior;
	   
	   // A $1 tenim una copia del simbol, hem de buscar l'original per canviar l'atribut 'complet'
	   Simbol *sym = obtenir_simbol(taulaSimbolActual, $1.nom);
	   
	   // Indicar que la funcio/procediment ja esta complet
	   if (sym != NULL) {
		   if (sym->atributs.tipus == PROCEDURE) {
			   int id = $1.atributs.continguts.info_procedure.id; // Identificador de sobrecarrega
			   
			   if (id > 0) {
				   // Busquem la sobrecarrega que correspon
				   reiniciar_PDI(sym->atributs.continguts.info_procedure.sobrecarregues);
				   int i;
				   for (i = 0; i < id; i++) {
						seguent_PDI(sym->atributs.continguts.info_procedure.sobrecarregues);
				   }
				   Info_Procedure *info_procedure = (Info_Procedure *)obtenir_element_PDI(sym->atributs.continguts.info_procedure.sobrecarregues);
				   info_procedure->complet = TRUE;
			   } else {
				   // Principal
				   sym->atributs.continguts.info_procedure.complet = TRUE;
			   }
			   
		   } else if (sym->atributs.tipus == FUNCTION) {
			   int id = $1.atributs.continguts.info_function.id; // Identificador de sobrecarrega
			   
			   if (id > 0) {
				   // Busquem la sobrecarrega que correspon
				   reiniciar_PDI(sym->atributs.continguts.info_function.sobrecarregues);
				   int i;
				   for (i = 0; i < id; i++) {
						seguent_PDI(sym->atributs.continguts.info_function.sobrecarregues);
				   }
				   Info_Function *info_function = (Info_Function *)obtenir_element_PDI(sym->atributs.continguts.info_procedure.sobrecarregues);
				   info_function->complet = TRUE;
			   } else {
				   // Principal
				   sym->atributs.continguts.info_function.complet = TRUE;
			   }
		   } else {
			   sinError("S'esperava PROCEDURE o FUNCTION.");
		   }
	   } else {
		   sinError("Simbol no trobat.");
	   }
   }
;

subprogram_body_element:
	subprogram_specification PR_IS {
		if ($1.atributs.tipus == PROCEDURE) {
		   taulaSimbolActual = $1.atributs.continguts.info_procedure.taula;
		} else if ($1.atributs.tipus == FUNCTION) {
			taulaSimbolActual = $1.atributs.continguts.info_function.taula;
		} else {
		   sinError("S'esperava PROCEDURE o FUNCTION. %i", $1.atributs.tipus);
		}
		memcpy(&$$, &$1, sizeof(struct Simbol));
	}
;

basic_declarative_item:
   basic_declaration { sinLog(" -> basic_declarative_item -> basic_declaration"); }
;

sequence_of_statements:
	statement sequence_of_statements_element { sinLog(" -> sequence_of_statements"); }
;

sequence_of_statements_element: /* Gramatica auxiliar de 'sequence_of_statements'*/
   statement {}
   | statement sequence_of_statements {}
   | {}
;

statement:
   simple_statement {}
   | compound_statement {}
;

simple_statement:
   null_statement {}
   | assignment_statement {}
   | procedure_call_statement {}
   | return_statement {}
;

null_statement:
   PR_NULL ';' { sinLog(" -> null_statement -> PR_NULL ';'"); }
;

assignment_statement: /* Comprovar que el id es una var i que esta declarada*/
   name PR_EQU expression ';' {
	   // TODO: Assignar el valor
	   
	   // A $1 rebem VARIABLE, si es un record hi trobarem la variable corresponent al element referenciat (e.g. date.month)
	   
	   if ($1.atributs.continguts.info_variable.constant == TRUE) {
		   sinError("No es pot utilitzar assignar un valor a una variable definida com a constant.");
	   }
	   
	   if ((($1.atributs.continguts.info_variable.tipus == TIPUS_DERIVAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT))
	   		|| (($1.atributs.continguts.info_variable.tipus == TIPUS_DERIVAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_DERIVAT))){
		   // Enumerat o ARRAY de enumerats
		   
		   // Comprovem si es array de enumerats
		   Simbol *symTipus = busca_totes_taules(taulaSimbolActual, $1.atributs.continguts.info_variable.nom_tipus);
		   if (symTipus->atributs.tipus == ARRAY_TYPE) {
			   
			   if (symTipus->atributs.continguts.info_array.tipus == TIPUS_DERIVAT) {
				   Simbol *symTipusElements = busca_totes_taules(taulaSimbolActual, symTipus->atributs.continguts.info_array.nom_tipus);
				   if (symTipusElements->atributs.tipus == ENUMERATED_TYPE) { 
					   // Array de enumerats
					   
					   // Comprovar que l'enumerat assignat pertany al tipus de la variable enumerada
					   Cua *elements = symTipusElements->atributs.continguts.info_enumerated.elements;
					   Boolean trobat = FALSE;
					   
					   reiniciar_PDI(elements);
					   Simbol *element = (Simbol *)obtenir_element_PDI(elements);
					   while(element != NULL) {
							// Comprovar parametres
							if (strcmp(element->nom, $3.nom) == 0) {
								trobat = TRUE;
								break;
							}
							
							seguent_PDI(elements);
							element = (Simbol *)obtenir_element_PDI(elements);
					   }
					   if (trobat == FALSE) {
						   sinError("La variable no es del mateix tipus ENUMERAT que el valor que s'intenta assignar.");
					   }
				   } else {
					   sinError("No s'espera aquest tipus.");
				   }
			   } else {
				   sinError("La variable no es del mateix tipus que el valor que s'intenta assignar.");
			   }
		   // sino... llavors es enumerat
		   } else if (strcmp($1.atributs.continguts.info_variable.nom_tipus, $3.atributs.continguts.info_variable.nom_tipus) != 0) {
				sinError("La variable no es del mateix tipus que el valor que s'intenta assignar.");
		   }
	   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) {
		   // Integer derivat
		   if (strcmp($1.atributs.continguts.info_variable.nom_tipus, $3.atributs.continguts.info_variable.nom_tipus) != 0) {
				sinError("La variable no es del mateix tipus que el valor que s'intenta assignar.");
		   }
		   
	   } else if (($1.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER)) {
		   // Integer derivat := Integer
		   
		   // Comprovació limits de rang pels tipus integer derivats
		   Simbol *symTipus = busca_totes_taules(taulaSimbolActual, $1.atributs.continguts.info_variable.nom_tipus);
		   if (symTipus->atributs.tipus == TYPE) {
			   if (($3.atributs.continguts.info_variable.valor.integer < (int)symTipus->atributs.continguts.info_type.rang.min)
					|| ($3.atributs.continguts.info_variable.valor.integer > (int)symTipus->atributs.continguts.info_type.rang.max)) {
							sinError("Valor fora de rang, ha d'estar entre %i i %i.", (int)symTipus->atributs.continguts.info_type.rang.min, (int)symTipus->atributs.continguts.info_type.rang.max);
				}
		   } else {
			   sinError("No es pot assignar un valor a un element d'un enumerat.");
		   }
	   } else if ($1.atributs.continguts.info_variable.tipus == TIPUS_DERIVAT) {
		   // Assignació a una posicio d'un ARRAY
		   
		   // Comprovació limits de rang pels tipus integer derivats
		   Simbol *symTipus = busca_totes_taules(taulaSimbolActual, $1.atributs.continguts.info_variable.nom_tipus);
		   if (symTipus->atributs.tipus == ARRAY_TYPE) {
			   //sinError("** %s - %i - %i", symTipus->nom, symTipus->atributs.continguts.info_array.tipus, $3.atributs.continguts.info_variable.tipus);
			   // Array d'enumerat ja ha sigut tractat en un cas anterior
			   if ((symTipus->atributs.continguts.info_array.tipus == TIPUS_INTEGER_DERIVAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) {
				   // Array d'integer derivat := integer derivat
				   if (strcmp(symTipus->atributs.continguts.info_array.nom_tipus, $3.atributs.continguts.info_variable.nom_tipus) != 0) {
					   sinError("La variable no es del mateix tipus que el valor que s'intenta assignar.");
				   }
			   } if ((symTipus->atributs.continguts.info_array.tipus == TIPUS_INTEGER_DERIVAT) && ($3.atributs.continguts.info_variable.tipus == TIPUS_INTEGER)) {
				   // Array d'integer derivat := Integer
				   
				   // Comprovació limits de rang pels tipus integer derivats
				   Simbol *symTipusElements = busca_totes_taules(taulaSimbolActual, symTipus->atributs.continguts.info_array.nom_tipus);
				   if (symTipusElements->atributs.tipus == TYPE) {
					   if (($3.atributs.continguts.info_variable.valor.integer < (int)symTipusElements->atributs.continguts.info_type.rang.min)
							|| ($3.atributs.continguts.info_variable.valor.integer > (int)symTipusElements->atributs.continguts.info_type.rang.max)) {
									sinError("Valor fora de rang, ha d'estar entre %i i %i.", (int)symTipusElements->atributs.continguts.info_type.rang.min, (int)symTipusElements->atributs.continguts.info_type.rang.max);
						}
				   } else {
					   sinError("??");
				   }
				   
				   
			   } else if (symTipus->atributs.continguts.info_array.tipus == TIPUS_DERIVAT) {
				   // ARRAY, RECORD
				   sinError("No poden haver arrays d'arrays ni arrays de records");
			   } else if (symTipus->atributs.continguts.info_array.tipus == $3.atributs.continguts.info_variable.tipus) {
				   // OK: Integer, Boolean, Float, String, Character
			   } else {
				   sinError("La variable no es del mateix tipus que el valor que s'intenta assignar.");
			   }
		   } else {
			   sinError("No s'espera altra cosa que un ARRAY_TYPE aqui.");
		   }
		    
	   } else if ($1.atributs.continguts.info_variable.tipus == $3.atributs.continguts.info_variable.tipus) {
		   // OK: Integer, Boolean, Float, String, Character
	   } else {
		   sinError("La variable no es del mateix tipus que el valor que s'intenta assignar.");
	   }

   }
;

procedure_call_statement:
   PR_ID_PROCEDURE {
	    Boolean trobat = FALSE;
		Info_Procedure *info_procedure;
		
		info_procedure = &($1->atributs.continguts.info_procedure);
		if (info_procedure->num_parametres == 0) {
			trobat = TRUE;
		}
		
		if (trobat == FALSE) {
			if ($1->atributs.continguts.info_procedure.sobrecarregues == NULL) {
				sinError("El procediment '%s' requereix '%i' paràmetre(s).", $1->nom, $1->atributs.continguts.info_procedure.num_parametres);
			} else {
				// Recorrer sobrecarregues
				reiniciar_PDI($1->atributs.continguts.info_procedure.sobrecarregues);
				info_procedure = (Info_Procedure *)obtenir_element_PDI($1->atributs.continguts.info_procedure.sobrecarregues);
				while(info_procedure != NULL) {
					// Comprovar parametres
					if (info_procedure->num_parametres == 0) {
						trobat = TRUE;
						break;
					}
					
					seguent_PDI($1->atributs.continguts.info_procedure.sobrecarregues);
					info_procedure = (Info_Procedure *)obtenir_element_PDI($1->atributs.continguts.info_procedure.sobrecarregues);
				}
			}
		}
		
		if (trobat) {
			// NOTE: Es pot cridar un procediment amb cos no definit si la signatura si ha sigut definida
			// Al final del compilador ja es comprova que tots tenen cos definit.
			/*if (info_procedure->complet == FALSE) {
				sinError("El procediment '%s' no te encara el cos definit.", $1->nom);
			}*/
			
			// TODO: Quin simbol retornem?
			/*$$.atributs.tipus = VARIABLE;
			$$.atributs.continguts.info_variable.tipus = info_procedure->tipus_retorn;
			$$.atributs.continguts.info_variable.literal = FALSE;
			$$.atributs.continguts.info_variable.inicialitzat = FALSE;
		
			if (($1->atributs.continguts.info_procedure.tipus_retorn == TIPUS_DERIVAT)
				|| ($1->atributs.continguts.info_procedure.tipus_retorn == TIPUS_INTEGER_DERIVAT)) {
				memcpy($$.atributs.continguts.info_variable.nom_tipus, info_procedure->nom_tipus_retorn, STRING_LENGTH);
			}*/
		} else {
			sinError("El procediment '%s' no te cap sobrecarrega que permeti '0' paràmetres.", $1->nom);
		}
   } // PROCEDURE
   
   | PR_ID_PROCEDURE actual_parameter_part ';' {
	    Boolean trobat = FALSE;
		Info_Procedure *info_procedure;
		
		info_procedure = &($1->atributs.continguts.info_procedure);
		// Mateix nombre de parametres?

		if (info_procedure->num_parametres == contador) {
			// Comprovar tipus parametres
			if (crida_parametres_diferents(cua_tmp, info_procedure->parametres, info_procedure->taula) == FALSE) {
				// Coincideixen el tipus dels parametres amb la definicio
				trobat = TRUE;
			}
		}
		
		if (trobat == FALSE) {
			if ($1->atributs.continguts.info_procedure.sobrecarregues == NULL) {
				sinError("El procediment '%s' requereix '%i' paràmetre(s) i no s'han trobat cap tipus coincident.", $1->nom, $1->atributs.continguts.info_procedure.num_parametres);
			} else {
				// Recorrer sobrecarregues
				reiniciar_PDI($1->atributs.continguts.info_procedure.sobrecarregues);
				info_procedure = (Info_Procedure *)obtenir_element_PDI($1->atributs.continguts.info_procedure.sobrecarregues);
				while(info_procedure != NULL) {
					// Mateix nombre de parametres?
					if (info_procedure->num_parametres == contador) {
						// Comprovar tipus parametres
						if (crida_parametres_diferents(cua_tmp, info_procedure->parametres, info_procedure->taula) == FALSE) {
							// Coincideixen el tipus dels parametres amb la definicio
							trobat = TRUE;
						}
						break;
					}
					
					seguent_PDI($1->atributs.continguts.info_procedure.sobrecarregues);
					info_procedure = (Info_Procedure *)obtenir_element_PDI($1->atributs.continguts.info_procedure.sobrecarregues);
				}
			}
		}
		
		if (trobat) {
			// NOTE: Es pot cridar un procediment amb cos no definit si la signatura si ha sigut definida
			// Al final del compilador ja es comprova que tots tenen cos definit.
			/*if (info_procedure->complet == FALSE) {
				sinError("El procediment '%s' no te encara el cos definit.", $1->nom);
			}*/
			
			cua_tmp = NULL;
			
			// TODO: Quin simbol retornem?
			/*$$.atributs.tipus = VARIABLE;
			$$.atributs.continguts.info_variable.tipus = info_procedure->tipus_retorn;
			$$.atributs.continguts.info_variable.literal = FALSE;
			$$.atributs.continguts.info_variable.inicialitzat = FALSE;
		
			if (($1->atributs.continguts.info_procedure.tipus_retorn == TIPUS_DERIVAT)
				|| ($1->atributs.continguts.info_procedure.tipus_retorn == TIPUS_INTEGER_DERIVAT)) {
				memcpy($$.atributs.continguts.info_variable.nom_tipus, info_procedure->nom_tipus_retorn, STRING_LENGTH);
			}*/
		} else {
			sinError("El procediment '%s' no te cap sobrecarrega que permeti '%i' paràmetre(s) amb aquests tipus.", $1->nom, contador);
		}
   }
;

actual_parameter_part:
  '(' {
		cua_tmp = (Cua *)crear_cua();
		contador = 0;
   }
   actual_parameter_part_element ')' {}
;

return_statement:
   PR_RETURN ';' {
	   Simbol *symRutina = busca_totes_taules(taulaSimbolActual, taulaSimbolActual->nom);
	   if (symRutina->atributs.tipus == FUNCTION) {
		   sinError("La funció ha de retornar un valor.");
	   }
   }
   | PR_RETURN expression ';' {
	   Simbol *symRutina = busca_totes_taules(taulaSimbolActual, taulaSimbolActual->nom);
	   if (symRutina->atributs.tipus != FUNCTION) {
		   sinError("No es possible ficar un RETURN amb valor a un procediment.");
	   }
	   
	   // S'ha especificat un return al cos de la funció
	   // No es pot saber en quina sobrecarrega estem, per tant millor utilitzar una variable global
	   // que sempre apunta al info_function actual.
	   //symRutina->atributs.continguts.info_function.return_indicat = TRUE;
	   infoFunctionActual->return_indicat = TRUE;
	   
	   if ((infoFunctionActual->tipus_retorn == TIPUS_DERIVAT) && ($2.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) {
		   // Enumerat := Element enumerat
			if (strcmp(infoFunctionActual->nom_tipus_retorn, $2.atributs.continguts.info_variable.nom_tipus) != 0) {
				sinError("La variable no es del mateix tipus que el valor que s'intenta assignar.");
		   }
	   } else if ((infoFunctionActual->tipus_retorn == TIPUS_DERIVAT) && ($2.atributs.continguts.info_variable.tipus == TIPUS_DERIVAT)) {
		   // Array, enumerat, record
		   if (strcmp(infoFunctionActual->nom_tipus_retorn, $2.atributs.continguts.info_variable.nom_tipus) != 0) {
				sinError("Error al valor de retorn, el tipus no coincideix.");
		   }
	   } else if ((infoFunctionActual->tipus_retorn == TIPUS_INTEGER_DERIVAT) && ($2.atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) {
		   // Integer derivat
		   if (strcmp(infoFunctionActual->nom_tipus_retorn, $2.atributs.continguts.info_variable.nom_tipus) != 0) {
				sinError("Error al valor de retorn, el tipus no coincideix.");
		   }
	   } else if ((infoFunctionActual->tipus_retorn == TIPUS_INTEGER_DERIVAT) && ($2.atributs.continguts.info_variable.tipus == TIPUS_INTEGER)) {
		   // Integer derivat := Integer
		   Simbol *symTipus = busca_totes_taules(taulaSimbolActual, infoFunctionActual->nom_tipus_retorn);
		   if (symTipus->atributs.tipus == TYPE) {
			   if (($2.atributs.continguts.info_variable.valor.integer < (int)symTipus->atributs.continguts.info_type.rang.min)
					|| ($2.atributs.continguts.info_variable.valor.integer > (int)symTipus->atributs.continguts.info_type.rang.max)) {
							sinError("Valor de retorn fora de rang, ha d'estar entre %i i %i.", (int)symTipus->atributs.continguts.info_type.rang.min, (int)symTipus->atributs.continguts.info_type.rang.max);
				}
		   } else {
			   sinError("Error al valor de retorn, el tipus no coincideix.");
		   }
	   } else if (infoFunctionActual->tipus_retorn == $2.atributs.continguts.info_variable.tipus) {
		   // OK: Integer, Boolean, Float, String, Character
	   } else {
		   sinError("Error al valor de retorn, el tipus no coincideix.");
	   }
   }
;

compound_statement:
   if_statement {}
   | loop_statement {}
;

if_statement:
   PR_IF condition PR_THEN sequence_of_statements PR_END PR_IF ';' {
	   sinLog(" -> if_statement -> PR_IF condition PR_THEN sequence_of_statements PR_END PR_IF ';'");
   }
   | PR_IF condition PR_THEN sequence_of_statements PR_ELSE sequence_of_statements PR_END PR_IF ';' {
	   sinLog(" -> if_statement -> PR_IF condition PR_THEN sequence_of_statements PR_ELSE sequence_of_statements PR_END PR_IF ';'");
   }
;

condition:
   expression {
	   sinLog(" -> condition -> expression");
	   
	   // El tipus ha de ser boolea
	   if ($1.atributs.tipus != VARIABLE) {
		   sinError("Condició no valida.");
	   }
	   if ($1.atributs.continguts.info_variable.tipus != TIPUS_BOOLEAN) {
		   sinError("Condició no valida.");
	   }
	   
	   memcpy(&$$, &$1, sizeof(struct Simbol));
   }
   /* boolean_expression {}*/
;

loop_statement:
   iteration_scheme PR_LOOP sequence_of_statements PR_END PR_LOOP ';' {}
;

iteration_scheme:
   PR_WHILE condition {
	   sinLog(" -> iteration_scheme -> PR_WHILE condition");
   }
   | PR_FOR loop_parameter_specification {
	   sinLog(" -> iteration_scheme -> PR_FOR loop_parameter_specification");
   }
;

loop_parameter_specification:
   PR_ID_VARIABLE PR_IN discrete_subtype_definition {
	   // TODO: Comprovar tipus de la variable amb el range
	   
	   if ($3.atributs.continguts.info_array.rang.tipus != TIPUS_INTEGER) {
		   sinError("No es permeten FOR amb valors diferents d'integer.");
	   }
	   
	   if ($1->atributs.continguts.info_variable.tipus == TIPUS_INTEGER) {
		   Simbol *symTipus = busca_totes_taules(taulaSimbolActual, "integer");
		   
		   if (($3.atributs.continguts.info_array.rang.min < (int)symTipus->atributs.continguts.info_type.rang.min)
				   || ($3.atributs.continguts.info_array.rang.max > (int)symTipus->atributs.continguts.info_type.rang.max)) {
		   		sinError("La variable no pot contenir valors definits al rang del FOR.");
		   }
	   } else if ($1->atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT) {
		   Simbol *symTipus = busca_totes_taules(taulaSimbolActual, $1->atributs.continguts.info_variable.nom_tipus);
		   
		   if (symTipus->atributs.tipus == TYPE) {
			   if (($3.atributs.continguts.info_array.rang.min < (int)symTipus->atributs.continguts.info_type.rang.min)
					   || ($3.atributs.continguts.info_array.rang.max > (int)symTipus->atributs.continguts.info_type.rang.max)) {
					sinError("La variable no pot contenir valors definits al rang del FOR.");
			   }
		   } else {
			   // Posible element d'enumerador
			   sinError("La variable d'un FOR ha de ser de tipus 'integer' o derivat.");
		   }
	   } else {
		   sinError("La variable d'un FOR ha de ser de tipus 'integer' o derivat.");
	   }
	   
	   if ($1->atributs.continguts.info_variable.constant == TRUE) {
		   sinError("No es pot utilitzar una variable definida com a constant en un FOR.");
	   }
	   
   }
;


program:
   program_element PR_IS declarative_part PR_BEGIN sequence_of_statements PR_END ';' {
	   sinLog(" -> program -> program_element PR_IS declarative_part PR_BEGIN sequence_of_statements PR_END ';'");
	   // NOTE: Comentar a l'informe que es comprova que existeixin tots els body de funcions i procediments
	   taulaSimbolActual = taulaSimbolActual->anterior;
	   
	   // Comprovar que tota funcio/procediment tingui body
	   comprovacio_complets(taulaSimbolActual);
   }
   | program_element PR_IS declarative_part PR_BEGIN sequence_of_statements PR_END designator ';' {
	   sinLog(" -> program -> program_element PR_IS declarative_part PR_BEGIN sequence_of_statements PR_END designator ';'");
	   taulaSimbolActual = taulaSimbolActual->anterior;
	   if (strcmp($1.nom, $7.nom) != 0) {
		   sinError("Designator '%s' no es correspon amb el nom de la funció/procediment %s.", $7.nom, $1.nom);
	   }
	   
	   // Comprovar que tota funcio/procediment tingui body
	   comprovacio_complets(taulaSimbolActual);
   }
;

program_element:
	PR_PROCEDURE defining_program_unit_name {
		Simbol *sym = crear_i_inserir_en_taula(taulaSimbolActual, $2);
		sym->atributs.tipus = PROCEDURE;
		sym->atributs.continguts.info_procedure.id = 0;
		sym->atributs.continguts.info_procedure.num_sobrecarregues = 0;
		sym->atributs.continguts.info_procedure.complet = TRUE;
		sym->atributs.continguts.info_procedure.num_parametres = 0;
		sym->atributs.continguts.info_procedure.parametres = NULL;
		sym->atributs.continguts.info_procedure.sobrecarregues = NULL;
		Taula_Simbols *ts = (Taula_Simbols *)crear_taula_simbols($2, taulaSimbolActual);
		sym->atributs.continguts.info_procedure.taula = ts;
		taulaSimbolActual = ts;
		
		memcpy(&$$, sym, sizeof(struct Simbol));
	}
;

%%

// Cerca un simbol a una cua de simbols, si no existeix retorna NULL, sino retorna el simbol
Simbol *cercar_simbol_a_cua(Cua *cua, char *nom) {
	reiniciar_PDI(cua);
	Simbol *element = (Simbol *)obtenir_element_PDI(cua);
	while(element != NULL) {
		if (strcmp(element->nom, nom) == 0) {
			return element;
		}		
		seguent_PDI(cua);
		element = (Simbol *)obtenir_element_PDI(cua);
	}
	return NULL;
}

void init_sintactic (char *file)
{
    char temp[100];
    strcpy(temp,file);
    strcat(temp,".sin.log");
    logsSintactic=fopen(temp,"w");
    if (logsSintactic==NULL)
        fprintf(stderr,"The sintactical analizer log file has not been created\n");
}

void end_sintactic(void)
{
    fclose(logsSintactic);
}


yyerror(char *s) {
    printf ("%s a la linia %i columna %i\n", s, lineNumber, colNumber);
    exit(0);
}


void sinError(const char *format, ...){
    va_list arg;
    char linea[4096];
    va_start(arg,format);
    sprintf(linea,"Error a la linia %i columna %i: ", lineNumber, colNumber);
    vsprintf(&linea[strlen(linea)],format,arg);
    sinLog("%s",linea);
    fprintf(stderr,"%s\n",linea);
    va_end(arg);
    exit(0);
}

void sinLog(const char *format, ...){
    va_list arg;
    char linea[4096];
    va_start(arg,format);
    if (logsSintactic!=NULL){
        vsprintf(linea,format,arg);
        fprintf(logsSintactic,"%s",linea);
        fprintf(logsSintactic,"\n");
    }
    va_end(arg);
}

// Duplica taula de simbols només si no conté simbols "dificils" de duplicar
// Útil per variables declarades RECORD, dupliquem la TS dels elements del RECORD que
// en teoria només han de ser del tipus VARIABLE i ARRAY_TYPE (arrays anonims).
// Si no pot duplicar retorna NULL
Taula_Simbols *intentar_duplicar_taula_simbols (Taula_Simbols *ts){
	int i;
	Simbol *simbol_dup, *simbol;
	Taula_Simbols *taula_dup = (Taula_Simbols *)crear_taula_simbols("DUPLICAT", NULL);
	
	for (i = 0; i < TAM_TAULA_SIMBOLS; i++) {
		if (ts->taula[i] != NULL) {
			Cua *colisions = ts -> taula[i];
			reiniciar_PDI(colisions);
			simbol = (Simbol *)obtenir_element_PDI(colisions);
			while(simbol != NULL) {
				if (simbol->atributs.tipus == VARIABLE) {
					simbol_dup = (Simbol *)malloc(sizeof(struct Simbol));
					memcpy(simbol_dup, simbol, sizeof(struct Simbol));
					
					if (simbol->atributs.continguts.info_variable.valor.taula != NULL) {
						// Variable d'un tipus RECORD i per tant te també subvariable
						simbol_dup->atributs.continguts.info_variable.valor.taula = intentar_duplicar_taula_simbols(simbol->atributs.continguts.info_variable.valor.taula);
					}
					inserir_simbol_en_taula(taula_dup, simbol_dup);
				} else if (simbol->atributs.tipus == TYPE) {
					simbol_dup = (Simbol *)malloc(sizeof(struct Simbol));
					memcpy(simbol_dup, simbol, sizeof(struct Simbol));
					inserir_simbol_en_taula(taula_dup, simbol_dup);
				} else if (simbol->atributs.tipus == RECORD_TYPE) {
					// NOTE: Conte taula de simbols, no permetem la duplicació
					return NULL;
				} else if (simbol->atributs.tipus == ARRAY_TYPE) {
					simbol_dup = (Simbol *)malloc(sizeof(struct Simbol));
					memcpy(simbol_dup, simbol, sizeof(struct Simbol));
					inserir_simbol_en_taula(taula_dup, simbol_dup);
				} else if (simbol->atributs.tipus == ENUMERATED_TYPE) {
					// NOTE: Conte cua d'elements, no permetem la duplicació
					return NULL;
				} else if (simbol->atributs.tipus == PROCEDURE) {
					// NOTE: Conte taula de simbols, no permetem la duplicació
					return NULL;
				} else if (simbol->atributs.tipus == FUNCTION) {
					// NOTE: Conte taula de simbols, no permetem la duplicació
					return NULL;
				}
				
				seguent_PDI(colisions);
				simbol = (Simbol *)obtenir_element_PDI(colisions);
			}
			return taula_dup;
		}
	}
}

// Espera rebre 2 cues d'strings de mateix tamany, 2 taules amb els simbols dels strings
// Retorna TRUE si les dues cues/taules son iguals, FALSE en cas contrari
// Util per comparar parametres i veure si el procediment/funcio esta sobrecarregat
Boolean parametres_diferents(Cua *strings_parametres1, Cua *strings_parametres2, Taula_Simbols *parametres1, Taula_Simbols *parametres2) {
	reiniciar_PDI(strings_parametres1);
	reiniciar_PDI(strings_parametres2);
	char *parametre1 = (char *)obtenir_element_PDI(strings_parametres1);
	char *parametre2 = (char *)obtenir_element_PDI(strings_parametres2);
	Boolean algun_nom_diferent = FALSE;
	Boolean tipus_diferent = FALSE;
	
	while (parametre1 != NULL) {
	   if (strcmp(parametre1, parametre2) != 0) {
		   algun_nom_diferent = TRUE;
	   }
	   
	   Simbol *sym_parametre1 = (Simbol *)obtenir_simbol(parametres1, parametre1);
	   Simbol *sym_parametre2 = (Simbol *)obtenir_simbol(parametres2, parametre2);
	   if (sym_parametre1->atributs.continguts.info_variable.tipus != sym_parametre2->atributs.continguts.info_variable.tipus) {
		   tipus_diferent = TRUE;
		   break;
	   }
	   
	   if ((sym_parametre1->atributs.continguts.info_variable.tipus == TIPUS_DERIVAT)
	   		|| (sym_parametre1->atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) {
		   if (strcmp(sym_parametre1->atributs.continguts.info_variable.nom_tipus, sym_parametre2->atributs.continguts.info_variable.nom_tipus) != 0) {
			   tipus_diferent = TRUE;
			   break;
		   }
	   }
	   
	   seguent_PDI(strings_parametres1);
	   seguent_PDI(strings_parametres2);
	   parametre1 = (char *)obtenir_element_PDI(strings_parametres1);
	   parametre2 = (char *)obtenir_element_PDI(strings_parametres2);
	}
	

	if (tipus_diferent == TRUE) {
		// Tipus diferents, no importa si hi ha algun nom que coincideix
		return TRUE;
	} else if (algun_nom_diferent == TRUE) {
		// Tipus iguals amb noms diferents. Lo important son els tipus, per tant considerem que NO son diferents..
		sinError("No es pot definir una funcio/procediment amb els mateixos tipus de parametres d'una altra ja definida, pero amb noms diferents.");
		//return TRUE;
	} else {
		// Tipus diferents i noms diferents
		return FALSE;
	}
		
}

// Espera rebre 1 cua de simbols VARIABLE, 1 cua d'strings de mateix tamany, 1 taula amb els simbols dels strings
// Retorna TRUE si les VARIABLEs son del mateix tipus que els parametres, FALSE en cas contrari
// Util per comparar parametres de la funcio/procediment i la seva crida
Boolean crida_parametres_diferents(Cua *valors, Cua *strings_parametres, Taula_Simbols *parametres) {
	reiniciar_PDI(valors);
	reiniciar_PDI(strings_parametres);
	Simbol *sym_valor = (Simbol *)obtenir_element_PDI(valors);
	char *parametre = (char *)obtenir_element_PDI(strings_parametres);
	Boolean diferent = FALSE;
	while (parametre != NULL) {

	   Simbol *sym_parametre = (Simbol *)obtenir_simbol(parametres, parametre);
	   if (sym_valor->atributs.continguts.info_variable.tipus != sym_parametre->atributs.continguts.info_variable.tipus) {
		   diferent = TRUE;
		   break;
	   }
	   
	   if ((sym_parametre->atributs.continguts.info_variable.tipus == TIPUS_DERIVAT)
	   			|| (sym_parametre->atributs.continguts.info_variable.tipus == TIPUS_INTEGER_DERIVAT)) {
		   
	   		if (strcmp(sym_valor->atributs.continguts.info_variable.nom_tipus, sym_parametre->atributs.continguts.info_variable.nom_tipus) != 0) {
			   diferent = TRUE;
			   break;
			}
	   }
	   
	   seguent_PDI(valors);
	   seguent_PDI(strings_parametres);
	   sym_valor = (Simbol *)obtenir_element_PDI(valors);
	   parametre = (char *)obtenir_element_PDI(strings_parametres);
	}
	
	return diferent;
}

// Comprova de forma recursiva, que tots els procediments/funcions tinguin body definit
// es a dir, siguin 'complets'.
void comprovacio_complets(Taula_Simbols *ts) {
	int i;
	
	for (i = 0; i < TAM_TAULA_SIMBOLS; i++) {
		if (ts->taula[i] != NULL) {
			Cua *colisions = ts -> taula[i];
			reiniciar_PDI(colisions);
			Simbol *simbol = (Simbol *)obtenir_element_PDI(colisions);
			while(simbol != NULL) {
				// EL SIMBOL ES UN PROCEDIMENT
				if (simbol->atributs.tipus == PROCEDURE) {
					if (simbol->atributs.continguts.info_procedure.complet == FALSE) {
						sinError("El procediment '%s' no te un cos definit.", simbol->nom);
					}
					// Comprovar procediments/funcions encaixades
					comprovacio_complets(simbol->atributs.continguts.info_procedure.taula);
					
					
					// Comprovar que totes les sobrecarregues tenen body
					if (simbol->atributs.continguts.info_procedure.num_sobrecarregues>0) {
						
						Cua *sobrecarregues = simbol->atributs.continguts.info_procedure.sobrecarregues;
						reiniciar_PDI(sobrecarregues);
						Info_Procedure *info_procedure = (Info_Procedure *)obtenir_element_PDI(sobrecarregues);
						while (info_procedure != NULL) {

							if (info_procedure->complet == FALSE) {
								sinError("El procediment sobrecarregat '%s' (%i) no te un cos definit.", simbol->nom, info_procedure->id);
							}
							// Comprovar procediments/funcions encaixades
							comprovacio_complets(info_procedure->taula);
							
							seguent_PDI(sobrecarregues);
							info_procedure = (Info_Procedure *)obtenir_element_PDI(sobrecarregues);
						}
					}
				}
	
				// EL SIMBOL ES UNA FUNCIO
				if (simbol->atributs.tipus == FUNCTION){
					if (simbol->atributs.continguts.info_function.complet == FALSE) {
						sinError("La funció '%s' no te un cos definit.", simbol->nom);
					}
					if (simbol->atributs.continguts.info_function.return_indicat == FALSE) {
						sinError("La funció '%s' no retorna cap valor en cap moment (falta RETURN).", simbol->nom);
					}
					// Comprovar procediments/funcions encaixades
					comprovacio_complets(simbol->atributs.continguts.info_function.taula);
					
					// Comprovar que totes les sobrecarregues tenen body
					if (simbol->atributs.continguts.info_function.num_sobrecarregues>0) {
						Cua *sobrecarregues = simbol->atributs.continguts.info_function.sobrecarregues;
						reiniciar_PDI(sobrecarregues);
						Info_Function *info_function = (Info_Function *)obtenir_element_PDI(sobrecarregues);
						while (info_function != NULL) {
							
							if (info_function->complet == FALSE) {
								sinError("La funció sobrecarregada '%s' (%i) no te un cos definit.", simbol->nom, info_function->id);
							}
							if (info_function->return_indicat == FALSE) {
								sinError("La funció sobrecarregada '%s' (%i) no retorna cap valor en cap moment (falta RETURN).", simbol->nom, info_function->id);
							}
							// Comprovar procediments/funcions encaixades
							comprovacio_complets(info_function->taula);
							
							seguent_PDI(sobrecarregues);
							info_function = (Info_Function *)obtenir_element_PDI(sobrecarregues);
						}
					}
				}
	
				seguent_PDI(colisions);
				simbol = (Simbol *)obtenir_element_PDI(colisions);
			}
		}
	}
}
	
